unit AlertDlg;

interface
uses
  Windows, Messages, SysUtils, Classes,dialogs,graphics,ExtCtrls,Forms;

type
  TAlertCloseEvent=procedure(Sender:TObject;ModalResult:Integer) of object;
  TAlertDlg = class(TComponent)
  private
    FDlgType: TMsgDlgType;
    FMsgDlgButtons:TMsgDlgButtons;
    FCaption:String;
    FMsgInfo:string;
    FIcon:TIcon;
    FHelpCtx: Longint;
    FX, FY: Integer;
    FHelpFileName: TFileName;
    FOnClose:TAlertCloseEvent;
    FOnOpen:TNotifyEvent;
    FColor:TColor;
    { Private declarations }
  protected
    { Protected declarations }
    procedure SetIcon(Value:TIcon);
  public
     constructor Create(Aowner:TComponent); override;
     destructor Destroy; override;
     function ShowModal:Integer;overload;
     function ShowModal(Msg:string):Integer;overload;
     function ShowModal(Title,msg:string):Integer;overload;
    { Public declarations }
  published
    property DlgType: TMsgDlgType read FDlgType Write FDlgType default mtInformation;
    property Buttons:TMsgDlgButtons read FMsgDlgButtons Write FMsgDlgButtons default [MBok];
    property Caption:String read FCaption Write FCaption;
    property Color:TColor read FColor Write FColor default clBtnFace;
    Property MsgInfo:string read FMsgInfo Write FMsgInfo;
    property Icon:TIcon read FIcon Write SetIcon;
    property DlgPosX:Integer read FX Write FX default -1;
    property DlgPosY:Integer read FY Write FY default -1;
    property HelpContext: Longint read FHelpCtx Write FHelpCtx;
    property HelpFile:TFileName read FHelpFileName Write FHelpFileName;
    property OnOpen:TNotifyEvent read FOnOpen Write FOnOpen;
    property OnClose:TAlertCloseEvent read FOnClose Write FOnClose;

    { Published declarations }
  end;

procedure Register;

implementation

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

constructor TAlertDlg.Create(Aowner:TComponent);
begin
   inherited;
   FDlgType:=mtInformation;
   Buttons:=[mbOK];
   FIcon:=TIcon.Create;
   FX:=-1;
   FY:=-1;
   FColor:=clBtnFace;
end;
destructor TAlertDlg.Destroy;
begin
   FIcon.Free;
   inherited;
end;
procedure TAlertDlg.SetIcon(Value:TIcon);
begin
   FIcon.Assign(Value);
end;
function TAlertDlg.ShowModal(Msg:string):Integer;
var
   mes:string;
begin
   mes:=Self.MsgInfo;
   Self.MsgInfo:=msg;
   Result:=Self.ShowModal;
   Self.MsgInfo:=mes;
end;
function TAlertDlg.ShowModal(Title,msg:string):Integer;
var
   cap,mes:string;
begin
   cap:=self.Caption;
   mes:=Self.MsgInfo;
   Caption:=Title;
   MsgInfo:=Msg;
   Result:=ShowModal;
   Self.Caption:=cap;
   Self.MsgInfo:=mes;
end;
function  TAlertDlg.ShowModal:Integer;
const
  mcHorzMargin = 8;
  mcVertMargin = 8;
var
   Form:TForm;
   i:Integer;
   DialogUnits: TPoint;
   HorzMargin, VertMargin:Integer;
begin
   Result:=-1;
   if Assigned(FOnOpen) then
      FOnOpen(Self);
   Form:=CreateMessageDialog(Self.MsgInfo,DlgType,Buttons);
   Form.Color:=Color;
   try
      DialogUnits := GetAveCharSize(Form.Canvas);
      HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
      VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
      Form.Caption:=Self.Caption;
      Form.HelpContext := Self.HelpContext;
      Form.HelpFile := Self.HelpFile;
      if DlgPosX >= 0 then Form.Left := DlgPosX;
      if DlgPosY >= 0 then Form.Top := DlgPosY;
      if (DlgPosY < 0) and (DlgPosX < 0) then Form.Position := poScreenCenter;
      if DlgType=mtCustom then
      begin
         Form.Width:=Form.Width+Icon.Width+HorzMargin;
         Form.Height:=Form.Height+Icon.Height;
         with TImage.Create(Form) do
         begin
            Name := 'Image';
            Parent := Form;
         end;
         for i:=0 to Form.ControlCount-1 do
            with Form.Controls[i] do
               if Name='Image' then
               begin
                  TImage(Form.Controls[i]).Picture.Icon.Assign(Icon);
                  TImage(Form.Controls[i]).SetBounds(HorzMargin, VertMargin, TImage(Form.Controls[i]).Picture.Icon.Width, TImage(Form.Controls[i]).Picture.Icon.Height);
               end
               else
                  if DlgType=mtCustom then
                  begin
                     if Name='Message' then
                        Left:=Left+Icon.Width+HorzMargin;
                     if ClassName='TButton' then
                     begin
                        Top:=Top+Icon.Height;
                        Left:=Left+Icon.Width-HorzMargin
                     end;
                  end;
      end;
      Result:=Form.ShowModal;
   finally
      Form.Free;
      if Assigned(FOnClose) then
         FOnClose(Self,result);
   end;
end;
procedure Register;
begin
  RegisterComponents('i-vcl Dialogs', [TAlertDlg]);
end;

end.
