unit CalcEvent;

interface

uses
  Windows, Messages,typinfo, SysUtils, Classes,forms,dialogs,controls,funciones;

type
  TFormula=(fSUM,fAVG,fNone,fMUL);
  TOnCalcEvent=procedure(Sender : TObject; msg: TMessage) of object;
  TCalcEvent = class(TComponent)
  private
     FOrigWindowProc : TWndMethod;
     FOnCalcEvent:TOnCalcEvent;
     FForm:TScrollingWinControl;
     FControls:TControls;
     FThrowIt:Boolean;
     Procedure ControlSubClass(var Message:TMessage);
    { Private declarations }
  protected
     procedure Init(AOwner:TComponent);
     procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;

    { Protected declarations }
  public
     constructor Create(AOwner:TComponent);override;
     Destructor Destroy;override;
     function GetResultAVG:Double;
     function GetResultSUM:Double;
     function GetSUMPercent(value:real):Double;
     function GetPercentAdded(value:real):Double;
     function GetResultMUL:Double;
     Function GetSeparated(sep:char):String;
    { Public declarations }
  published
     Property OnCatchEvent:TOnCalcEvent read FOnCalcEvent write FOnCalcEvent;
     property Controls:TControls read FControls Write FControls;
    { Published declarations }
  end;

procedure Register;

implementation
constructor TCalcEvent.Create(AOwner:TComponent);
begin
   inherited;
   FOrigWindowProc:=nil;
   FForm:=AOwner as TScrollingWinControl;
   FControls:=TControls.Create(self);
   Init(AOwner);
   FThrowIt:=false;
end;
destructor TCalcEvent.Destroy;
begin
   if assigned(FOrigWindowProc) then
   begin
      fForm.WindowProc:=FOrigWindowProc;
      FOrigWindowProc:=nil;
   end;
   inherited;
end;
procedure TCalcEvent.Init(Aowner:TComponent);
begin
   if not (csDesigning in aowner.ComponentState) then
   begin
      FOrigWindowProc:= fform.WindowProc;
      fForm.WindowProc:=ControlSubClass;
   end;
end;
procedure TCalcEvent.Notification(AComponent: TComponent;
  Operation: TOperation);
var
   i:integer;
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
     for i:=0 to FControls.Count-1 do
        if (acomponent.Name=FControls[i].Control.Name) then
        begin
           Fcontrols.Delete(i);
           break;
        end;
end;
Procedure TCalcEvent.ControlSubClass(var Message:TMessage);
   function IsInControls(sender:TWinControl):Boolean;
   var
      i:integer;
      found:Boolean;
   begin
      found:=false;
      for i:=0 to Controls.Count-1 do
         if controls[i].Control.Name=Sender.Name then
         begin
            found:=true;
            break;
         end;
      result:=found;
   end;
var
   Throw:boolean;
begin
   Throw:=false;
   FOrigWindowProc(message);
   case message.Msg of
      CM_FOCUSCHANGED:
            if IsInControls(TCMFocusChanged(message).Sender) then
            begin
               Throw:=true;
               FThrowIt:=true;
            end
            else
            begin
               if FThrowIt then
                  Throw:=true;
               FThrowIt:=false;
            end;
      CM_CHANGED:
            if TCMChanged(message).Child.InheritsFrom(TWinControl) then
            if IsInControls(TWinControl(TCMChanged(message).Child)) then
            if not TWinControl(TCMChanged(message).Child).Focused then
               Throw:=true;
   end;
   if assigned(FOnCalcEvent) and Throw then
      FOnCalcEvent(self,message);
end;
function TCalcEvent.GetResultSUM:Double;
var
   a:variant;
   i:integer;
begin
   result:=0;
   if (controls.Count>0) then
      for i:=0 to controls.Count-1 do
        if IsPublishedProp(controls[i].Control,'Text') then
        begin
            a:=GetStrProp(controls[i].Control,'Text');
            try
               result:=result+a;
            except
            end;
        end;
end;
function TCalcEvent.GetSUMPercent(value:real):Double;
begin
   try
      result:=GetResultSUM*(value/100);
   except
      result:=0;
   end;
end;
function TCalcEvent.GetPercentAdded(value:real):Double;
begin
   try
      result:=GetResultSUM*(1+(value/100));
   except
      result:=0;
   end;
end;
function TCalcEvent.GetResultAVG:Double;
var
   a:variant;
   i,n:integer;
begin
   result:=0;
   if (controls.Count>0) then
   begin
      n:=0;
      for i:=0 to controls.Count-1 do
        if IsPublishedProp(controls[i].Control,'Text') then
        begin
           n:=n+1;
           a:=GetStrProp(controls[i].Control,'Text');
           try
              result:=result+a;
           except
           end;
           result:=result/n;
        end;
   end;
end;
function TCalcEvent.GetResultMUL:Double;
var
   a:variant;
   i:integer;
   n:integer;
begin
   result:=0;
   n:=0;
   if (controls.Count>0) then
   begin
      result:=1;
      for i:=0 to controls.Count-1 do
         if IsPublishedProp(controls[i].Control,'Text') then
         begin
            a:=GetStrProp(controls[i].Control,'Text');
            try
               result:=result*a;
               n:=1;
            except
            end;
         end;
   end;
   if n=0 then result:=0;
end;
function TCalcEvent.GetSeparated(sep:char):String;
var
   a:variant;
   i:integer;
begin
   result:='';
   for i:=0 to controls.Count-1 do
     if IsPublishedProp(controls[i].Control,'Text') then
     begin
         a:=GetStrProp(controls[i].Control,'Text');
         result:=result+sep+a;
     end;
   if result<>'' then
      result:=copy(result,2,length(result)-2);
end;

procedure Register;
begin
  RegisterComponents('i-vcl Non-Visual', [TCalcEvent]);
end;

end.
