unit CatchEvent;

interface

uses
  Windows, Messages, SysUtils, Classes,forms,dialogs,controls,igriegaStrs,funciones;
type
  TMsgType=(mtAny,mtSetFocus,mtKillFocus,mtSetText,mtGetText,
            mtKeyDown,mtKeyUp,mtChar,
            mtPaste,mtCut,mtCopy,mtClear,
            mtHScroll,mtVScroll,
            mtMouseWheel,mtLButtonDown,mtLButtonUp,mtLButtonDblClk,
            mtRButtonDown,mtRButtonUp,mtRButtonDblClk,
            mtMButtonDown,mtMButtonUp,mtMButtonDblClk,
            mtMove,mtMouseEnter,mtMouseLeave,mtMouseMove,
            mtPaint);
  TCatch=(cAfter,cBefore);
  TMsgTypeS=set of TMsgType;
  TCatchEventsFrom=(ceForm,ceControl);
  TOnCatchEvent=procedure(Sender : TObject; msg: TMessage) of object;
  TCustomCatchEvent = class(TComponent)
  private
     FOnCatchEvent:TOnCatchEvent;
     FMsgType:TMsgTypeS;
     FCatch:TCatch;
    { Private declarations }
  protected
    { Protected declarations }
  public
     constructor Create(AOwner:TComponent);override;
    { Public declarations }
     Property OnCatchEvent:TOnCatchEvent read FOnCatchEvent write FOnCatchEvent;
     property MessageCatched:TMsgTypeS read FMsgType write FMsgType default [mtAny] ;
     property Catch:TCatch read FCatch write FCatch default cAfter;
    { Published declarations }
  end;

const
  SMsgType:array[low(TmsgType)..high(TmsgType)] of Cardinal =
         (0,WM_SETFOCUS,WM_KILLFOCUS,WM_SETTEXT,WM_GETTEXT,WM_KEYDOWN,WM_KEYUP,WM_CHAR,
          WM_PASTE,WM_CUT,WM_COPY,WM_CLEAR,
          WM_HSCROLL,WM_VSCROLL,
          WM_MOUSEWHEEL,WM_LBUTTONDOWN,WM_LBUTTONUP,WM_LBUTTONDBLCLK,
          WM_RBUTTONDOWN,WM_RBUTTONUP,WM_RBUTTONDBLCLK,
          WM_MBUTTONDOWN,WM_MBUTTONUP,WM_MBUTTONDBLCLK,
          WM_MOVE,CM_MOUSEENTER,CM_MOUSELEAVE,WM_MOUSEMOVE,
          WM_PAINT);
type
  TControlEvItem=class(TCollectionItem)
     private
        FControl:TControl;
        FOrigWindowProc : TWndMethod;
        procedure SetControl(value:TControl);
        Procedure ControlSubClass(var Message:TMessage);
     protected
        function GetDisplayName: string; override;
        property OrigWindowProc : TWndMethod read FOrigWindowProc write FOrigWindowProc;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Control:TControl read FControl Write SetControl;
  end;
  TControlsEv=Class(TCollection)
     private
        FOwner:TPersistent;
        FControlMsg:TControl;
        function GetItem(Index:Integer):TControlEvItem;
        Procedure SetItem(Index:Integer;Value:TControlEvItem);
     protected
        function GetOwner:TPersistent;override;
        procedure Notify(Item: TCollectionItem; Action: TCollectionNotification);override;
        property ControlMsg:TControl read FControlMsg write FControlMsg;
     public
         Constructor Create(Owner:TPersistent);
         Function Add:TControlEvItem;
         property Items[Index:Integer]:TControlEvItem read Getitem write SetItem; default;
     published
  end;

  TCatchEvent = class(TCustomCatchEvent)
  private
     FOrigWindowProc : TWndMethod;
     FCatchFrom:TCatchEventsFrom;
     FWindow:TControl;
     Procedure ControlSubClass(var Message:TMessage);
     procedure SetWindow(value:tControl);
     procedure SetCatchFrom(value:TCatchEventsFrom);
    { Private declarations }
  protected
     procedure Init(AOwner:TComponent);
     procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    { Protected declarations }
  public
     constructor Create(AOwner:TComponent);override;
     Destructor Destroy;override;
    { Public declarations }
  published
     property Window:TControl read FWindow Write SetWindow;
     property CatchFrom:TCatchEventsFrom read FCatchFrom write SetCatchFrom;
     Property OnCatchEvent;
     property MessageCatched;
     property Catch;
    { Published declarations }
  end;
  TMultiCatchEvent = class(TCustomCatchEvent)
     private
        FControls:TControlsEv;
     protected
        procedure Notification(AComponent: TComponent;
         Operation: TOperation); override;
     public
       constructor Create(AOwner:TComponent);override;
       Destructor Destroy;override;
     published
        Property OnCatchEvent;
        property MessageCatched;
        property Catch;
        property Controls:TControlsEv read FControls Write FControls;
  end;
procedure Register;

implementation

constructor TControlEvItem.Create;
begin
   inherited;
   FOrigWindowProc:=nil;
end;
Procedure TControlEvItem.ControlSubClass(var Message:TMessage);
var
   m:TMessage;
   i:tmsgtype;
   found:boolean;
begin
   TControlsEv(Collection).ControlMsg:=FControl;
   found:=false;
   m:=message;
   with TMultiCatchEvent(Collection.Owner) do
   begin
     if Catch=cAfter then
        FOrigWindowProc(message);
     if assigned(OnCatchEvent) then
        if mtAny in messageCatched then
           found:=true
        else
           for i:=low(i) to high(i) do
           begin
              if (i in messageCatched) and (m.Msg=SMsgType[i]) then
              begin
                 found:=true;
                 break;
              end;
           end;
     if found then OnCatchEvent(self.Control,m);
     if Catch=cBefore then FOrigWindowProc(message);
     end;
end;
Procedure TControlEvItem.SetControl(value:TControl);
var
   i:integer;
   found:boolean;
   procedure Asign;
   begin
      if not (csDesigning in TComponent(Collection.Owner).ComponentState) then
      begin
         FOrigWindowProc:=FControl.WindowProc;
         FControl.WindowProc:=self.ControlSubClass;
      end;
   end;
begin
  found:=false;
  if (value<>nil) and (fcontrol<>value) then
  begin
     for i:=0 to Collection.Count-1 do
        if assigned(TControlEvItem(Collection.Items[i]).Control) then
           if TControlEvItem(Collection.Items[i]).Control.Name=value.Name then
           begin
              raise Exception.CreateResFmt(@sControlExistErrMsg, [value.Name]);
              found:=true;
              break;
           end;
     if not found then
     begin
        FControl := value;
        Asign;
     end;
  end
  else
     if value<>nil then
     begin
        if assigned(FControl) then
        begin
           if not (csDesigning in TComponent(Collection.Owner).ComponentState) then
              FControl.WindowProc:=FOrigWindowProc;
           FControl := value;
           Asign;
        end
        else
        begin
           FControl := value;
           Asign;
        end;
     end
     else
        if assigned(FControl) then
        begin
           if not (csDesigning in TComponent(Collection.Owner).ComponentState) then
              FControl.WindowProc:=FOrigWindowProc;
           FControl:=value;
           FOrigWindowProc:=nil;
        end;
end;
function TControlEvItem.GetDisplayName: string;
begin
   if assigned(FControl) then
      result:=self.Control.Name
   else
       result:='Control'+inttostr(index);
end;
constructor TControlsEv.Create(Owner:TPersistent);
begin
   inherited Create(TControlEvItem);
   Fowner:=owner;
end;
procedure TControlsEv.Notify(Item: TCollectionItem; Action: TCollectionNotification);
begin
   if action=cnDeleting then
      if assigned(TControlEvItem(Item).OrigWindowProc) then
      begin
         TControlEvItem(Item).Control.WindowProc:=TControlEvItem(Item).OrigWindowProc;
         TControlEvItem(Item).OrigWindowProc:=nil;
      end;
   inherited Notify(Item, Action);
end;
function TControlsEv.GetItem(Index:Integer):TControlEvItem;
begin
   result:=TControlEvItem(inherited GetItem(Index));
end;
Procedure TControlsEv.SetItem(Index:Integer;Value:TControlEvItem);
begin
   inherited SetItem(Index,Value);
end;
Function TControlsEv.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TControlsEv.Add:TControlEvItem;
begin
   result:=TControlEvItem(Inherited Add);
end;

constructor TCustomCatchEvent.Create(AOwner:TComponent);
begin
   inherited;
   FMsgType:=[mtAny];
   FCatch:=cAfter;
end;

constructor TCatchEvent.Create(AOwner:TComponent);
begin
   inherited;
   FOrigWindowProc:=nil;
   FCatchFrom:=ceForm;
   FWindow:=AOwner as TScrollingWinControl;
end;
destructor TCatchEvent.Destroy;
begin
   if assigned(FOrigWindowProc) then
   begin
      fWindow.WindowProc:=FOrigWindowProc;
      FOrigWindowProc:=nil;
   end;
   inherited;
end;
procedure TCatchEvent.Init(Aowner:TComponent);
begin
   if not (csDesigning in aowner.ComponentState) then
   begin
      FOrigWindowProc:= fwindow.WindowProc;
      fwindow.WindowProc:=self.ControlSubClass;
   end;
end;
procedure TCatchEvent.SetCatchFrom(value:TCatchEventsFrom);
begin
   if FCatchFrom<>value then
   begin
      FCatchFrom:=value;
      if value=ceForm then
         window:=Owner as TScrollingWinControl
      else
         window:=nil;
   end;
end;
procedure TCatchEvent.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
     if (Window <> nil) and (acomponent.Name=Window.Name) then
        Window := nil;
end;
procedure TCatchEvent.SetWindow(value:tControl);
begin
   if assigned(FOrigWindowProc) then
   begin
      fWindow.WindowProc:=FOrigWindowProc;
      FOrigWindowProc:=nil;
   end;
   FWindow:=value;
   if value<>nil then
      init(self.Owner);
end;
Procedure TCatchEvent.ControlSubClass(var Message:TMessage);
var
   m:TMessage;
   i:tmsgtype;
   found:boolean;
begin
   found:=false;
   m:=message;
   if Catch=cAfter then
      FOrigWindowProc(message);
   if assigned(OnCatchEvent) then
      if mtAny in messageCatched then
         found:=true
      else
         for i:=low(i) to high(i) do
         begin
            if i in messageCatched then
               if m.Msg=SMsgType[i] then
               begin
                  found:=true;
                 break;
               end;
         end;
   if found then OnCatchEvent(Window,m);
   if Catch=cBefore then
      FOrigWindowProc(message);
end;
constructor TMultiCatchEvent.Create(AOwner:TComponent);
begin
   inherited;
   FControls:=TControlsEv.Create(self);
end;
destructor TMultiCatchEvent.Destroy;
begin
   FControls.Free;
   inherited;
end;

procedure TMultiCatchEvent.Notification(AComponent: TComponent;
  Operation: TOperation);
var
   i:integer;
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
     for i:=0 to Controls.Count-1 do
        if assigned(FControls[i].Control) then
           if (acomponent.Name=Controls[i].Control.Name) then
           begin
              controls.Items[i].Control:=nil;
              break;
           end;
end;

procedure Register;
begin
  RegisterComponents('i-vcl Non-Visual', [TCatchEvent]);
  RegisterComponents('i-vcl Non-Visual', [TMultiCatchEvent]);
end;

end.
