unit DBDateTimePicker;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ComCtrls,DB,DBCtrls,CommCtrl;

type
  TDBDateTimePicker = class(TDateTimePicker)
     private
     FDataLink:TFieldDataLink;
     function GetReadOnly: Boolean;
     procedure SetReadOnly(Value: Boolean);
     Function GetDataField:string;
     Procedure SetDataField(const Value:string);
     Function GetDataSource:TDataSource;
     procedure SetDataSource(Value:TDataSource);
     Procedure UpdateData(Sender:TObject);
     Procedure DataChange(Sender:TObject);
     procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
     procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
  protected
     procedure Change; override;
     Procedure Notification(AComponent:TComponent;Operation:TOperation); override;
  public
     Constructor Create(AOwner : TComponent);override;
     Destructor Destroy;override;
  published
     Property ReadOnly : Boolean read GetReadOnly write SetReadOnly default False;
     Property DataField : String read GetDataField write SetDataField;
     Property Datasource : TDataSource read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation
constructor TDBDateTimePicker.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   ControlStyle:=controlStyle-[csReplicatable];
   DateTime:=now;
   FDataLink:=TFieldDataLink.Create;
   FDataLink.OnDataChange:=DataChange;
   FDataLink.OnUpdateData:=UpdateData;
   fdatalink.Control:=self;
   Checked:=False;
end;
destructor TDBDateTimePicker.destroy;
begin
   FDataLink.Free;
   FDataLink:=nil;
   inherited Destroy;
end;
procedure TDBDateTimePicker.CNNotify(var Message: TWMNotify);
begin
  inherited;
  with Message, NMHdr^ do
  begin
    Result := 0;
    case code of
      DTN_CLOSEUP:change;
      DTN_DATETIMECHANGE:;
      DTN_DROPDOWN:;
      DTN_USERSTRING:;
    end;
  end;
end;

function TDBDateTimePicker.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TDBDateTimePicker.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TDBDateTimePicker.Change;
begin
   if assigned(FDataLink.field) then
     if not FDataLink.field.ReadOnly then
     begin
        FDataLink.DataSource.Edit;
        inherited Change;
     end
     else
        self.DataChange(self);
end;
function TDBDateTimePicker.GetDataField:String;
begin
   Result:=FDataLink.FieldName;
end;
procedure TDBDateTimePicker.SetDataField(const value:string);
begin
   FDataLink.FieldName:=Value;
end;
function TDBDateTimePicker.GetDataSource:TDataSource;
begin
   Result:=FDataLink.DataSource;
end;
Procedure TDBDateTimePicker.SetDataSource(value:tdatasource);
begin
   if FDataLink.DataSource<>Value then
   begin
      FDataLink.DataSource:=Value;
      If Value<>nil then
         Value.FreeNotification(self);
   end;
end;
Procedure TDBDateTimePicker.Notification(AComponent:TComponent;Operation:TOperation);
begin
   inherited Notification(Acomponent,Operation);
   if (Operation=opRemove) AND (FDataLink<>nil) and (AComponent=DataSource) then
      DataSource:=nil;
end;
Procedure TDBDateTimePicker.DataChange(Sender : TObject);
begin
   if not assigned(FDataLink.Field) then
      self.Date:=now
   else
      if FdataLink.Field.AsDateTime>0.0 then
         self.DateTime:=FdataLink.Field.AsDateTime
      else self.Date:=now;
end;
Procedure TDBDateTimePicker.UpdateData(Sender : TObject);
begin
   if assigned(FDataLink.Field) then
   if not (fdatalink.field.ReadOnly or fdatalink.ReadOnly) then
   begin
      if not fdatalink.Editing then//not (DataSource.State in [dsInsert,dsEdit]) then
         DataSource.Edit;
      FdataLink.Field.AsDateTime:=self.DateTime;
      fdatalink.Modified;
   end;
end;
procedure TDBDateTimePicker.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

procedure Register;
begin
  RegisterComponents('i-vcl DB', [TDBDateTimePicker]);
end;

end.
