unit EditCheck;

interface

uses
  Windows, Messages, SysUtils,dialogs, Classes, Controls, StdCtrls,buttons,graphics;
const
  AlignText:array[taLeftJustify..taCenter] of DWORD =(ES_LEFT,ES_RIGHT,ES_CENTER);
type
  TSide=(sbLeft,sbRight);
  TCustomEditCheck = class(TCustomEdit)
  private
     FCheckBox:TCheckBox;
     FOnCheckBoxClick : TNotifyEvent;
     FWidthButton:Integer;
     FSide:TSide;
     FAlignment: TAlignment;
     FLetChange:Boolean;
     FOrigWindowProc : TWndMethod;
     FFocusedColor:TColor;
     FCheckEnabled:Boolean;
     procedure SetEditRect;
     procedure ClickButton(Sender: TObject);
     Procedure SetSide(Value:TSide);
     Procedure SetAlignment(value:TAlignment);
     procedure SetCheckEnabled(Value:Boolean);
     function GetCheckEnabled:Boolean;
     procedure WMSize(var Message: TWMSize); message WM_SIZE;
     procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
     procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
     Procedure ControlSubClass(var Message:TMessage);
    { Private declarations }
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetChecked(Value:Boolean);
    function GetChecked:Boolean;
    property OnCheckBoxClick : TNotifyEvent read FOnCheckBoxClick write FOnCheckBoxClick;
    property Side:TSide read FSide write SetSide default sbRight;
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Checked:Boolean read GetChecked Write SetChecked default false;
    procedure Change; override;
    property LetChange:Boolean read FLetChange Write FLetChange default False;
    property FocusedColor:TColor read FFocusedColor Write FFocusedColor default clBlack;
    property CheckEnabled:Boolean read GetCheckEnabled Write SetCheckEnabled default True;
    Property ReadOnly;
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;override;
  published
    { Published declarations }
  end;
  TEditCheck = class(TCustomEditCheck)
  published
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Checked;
    property CheckEnabled;
    property Color;
    property FocusedColor;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property LetChange;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property OnCheckBoxClick;
    property Side;
    property Alignment;
  end;
implementation

constructor TCustomEditCheck.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FAlignment:=taLeftJustify;
  FCheckBox:=TCheckBox.Create(self);
  FCheckBox.Width := 15;
  FCheckBox.Height := Height - 2;
  FCheckBox.Top:=1;
  FCheckBox.Left:=1;
  FCheckBox.Visible := True;
  FCheckBox.Parent := Self;
  FCheckBox.OnClick:=clickbutton;
  FCheckBox.Cursor:=crArrow;
  FOrigWindowProc:=FCheckBox.WindowProc;
  FCheckBox.WindowProc:=ControlSubClass;
  FSide:=sbRight;
  FWidthButton:=trunc((FCheckBox.Width*100/width));
  FLetChange:=False;
  FFocusedColor:=clBlack;
  FCheckEnabled:=True;
end;
destructor TCustomEditCheck.Destroy;
begin
  if assigned(FOrigWindowProc) then
     FCheckBox.WindowProc:=FOrigWindowProc;
   FCheckBox.Free;
   inherited;
end;
function TCustomEditCheck.GetCheckEnabled:Boolean;
begin
      Result:=FCheckBox.Enabled;
end;
procedure TCustomEditCheck.SetCheckEnabled(Value:Boolean);
begin
   if CheckEnabled<>Value then
   begin
      FCheckBox.Enabled:=Value;
   end;
end;
Procedure TCustomEditCheck.ControlSubClass(var Message:TMessage);
begin
   FOrigWindowProc(message);
   if Message.Msg=CM_FOCUSCHANGED then
      if  TCMFocusChanged(Message).Sender=FCheckBox then
         FCheckBox.Color:=FocusedColor
      else
       FCheckBox.Color:=Self.Color;
end;
procedure TCustomEditCheck.WMSetFocus(var Message: TWMSetFocus);
begin
   if Checked and not LetChange then
   BEGIN
      if CheckEnabled then
         FCheckBox.SetFocus;
//      Parent.Perform(WM_NEXTDLGCTL,0,0);
      Message.Result:=0;
   END
   else
      inherited;
end;

procedure TCustomEditCheck.SetChecked(Value:Boolean);
begin
   if Value<>FCheckBox.Checked then
      FCheckBox.Checked:=Value;
end;
function TCustomEditCheck.GetChecked:Boolean;
begin
   Result:=FCheckBox.Checked;
end;
procedure TCustomEditCheck.Change;
begin
   if not (csLoading in ComponentState) then
   if Checked then Checked:=False;
   inherited Change;
end;
procedure TCustomEditCheck.Clickbutton(sender:tobject);
begin
   if not (csDesigning in ComponentState) then
   begin
      if assigned(fOnCheckBoxClick) then
      begin
         fOnCheckBoxClick(sender);
      end;
   end;
end;
procedure TCustomEditCheck.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_CLIPCHILDREN or ES_MULTILINE  or AlignText[FAlignment] or ES_AUTOHSCROLL;
end;
procedure TCustomEditCheck.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure TCustomEditCheck.SetEditRect;
var
  Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  if fside=sbLeft then
  begin
     Loc.Bottom := ClientHeight + 1;
     Loc.Right := ClientWidth - 2;
     Loc.Top := 0;
     Loc.Left := FCheckBox.Width + 2;
  end
  else
  begin
     Loc.Bottom := ClientHeight + 1;
     Loc.Right := ClientWidth - 2 - FCheckBox.Width;
     Loc.Top := 0;
     Loc.Left := 0;
  end;
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
end;
procedure TCustomEditCheck.WMSize(var Message: TWMSize);
begin
  inherited;
  if ctl3d then
     FCheckBox.Height:=ClientHeight-2
  else
     FCheckBox.Height:=ClientHeight-2;
  SetEditRect;
end;
procedure TCustomEditCheck.WMPaint(var Message: TWMPaint);
var
   Mes : TWMPaint;
begin
  Mes:=Message;
  if FSide=sbLeft then
     FCheckBox.Left:=1
  else
     FCheckBox.Left:=(ClientWidth - 2 -FCheckBox.Width);
  if ctl3d then
     FCheckBox.Height:=ClientHeight-2 else
     FCheckBox.Height:=ClientHeight-2;
  SetEditRect;
  inherited;
end;
Procedure TCustomEditCheck.SetSide(Value:TSide);
begin
   if fside<>value then
   begin
      fside:=value;
      if value=sbLeft then
         FCheckBox.Left:=1
      else
         FCheckBox.Left:=(ClientWidth - 2 -FCheckBox.Width);
      invalidate;
      SetEditRect;
   end;
end;
Procedure TCustomEditCheck.SetAlignment(value:TAlignment);
begin
   if FAlignment<>value then
   begin
      FAlignment:=value;
      ReCreateWnd;
   end;
end;
end.
