unit EditDo;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,
  funciones, Editb,igriegaclasses,dialogs,igriegastrs;
type
  TLocType=(ltNone,ltLocate,ltContains,ltBeginWith);
  TEditDo = class(TCustomEditb)
  private
    FValueList:TStringsList;
    FIndexValue:integer;
    FCad:String;
    FLocType:TLocType;
    FTmp:string;
    Function GetValue:String;
    Function GetCaseSensitive:Boolean;
    Procedure SetCaseSensitive(Value:Boolean);
    Procedure SetDuplicates(Value:TDuplicates);
    Function GetDuplicates:TDuplicates;
    Procedure SetSorted(Value:Boolean);
    Function GetSorted:Boolean;
    Procedure SetIndexValue(Value:Integer);
    { Private declarations }
  protected
     procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
     procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    { Protected declarations }
  public
     constructor Create(AOwner: TComponent);override;
     Destructor Destroy;override;
    { Public declarations }
     procedure Add(Const Cad:string);
     function Locate(cad:string):Boolean;
     function Contains(cad:string):Boolean;
     function BeginWith(cad:string):Boolean;
     Function FindNext:Boolean;
     function Next:String;
     function Previous:String;
     property IndexValue:Integer read FIndexValue write SetIndexValue;
     property Value:string read GetValue;
     function GetListValues:TStringList;
  published
    property CaseSensitive:Boolean read GetCaseSensitive write SetCaseSensitive default false;
    Property Duplicates:TDuplicates read GetDuplicates write SetDuplicates default dupIgnore;
    Property Sorted:Boolean read GetSorted write SetSorted default false;
    property ButtonType;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property OnButtonClick;
    property TextButton;
    property WidthButton;
    property ImageButton;
    property FlatButton;
    property TransparentButton;
    property FontButton;
    property Side;
    property Alignment;
    property ButtonVisible;

    { Published declarations }
  end;
procedure Register;

implementation
constructor TEditDo.Create(AOwner: TComponent);
begin
   inherited;
   FLocType:=ltNone;
   FValueList:=TStringsList.Create;
   FIndexValue:=-1;
   FCad:='';
   FTmp:='';
end;
destructor TEditDo.Destroy;
begin
   FValueList.Free;
   inherited;
end;
Function TEditDo.GetCaseSensitive:Boolean;
begin
   result:=FValueList.CaseSensitive
end;
Procedure TEditDo.SetCaseSensitive(Value:Boolean);
begin
   if Value<>FValueList.CaseSensitive then
      FValueList.CaseSensitive:=Value;
end;
Procedure TEditDo.SetDuplicates(Value:TDuplicates);
begin
   if Value<>FValueList.Duplicates then
      FValueList.Duplicates:=Value;
end;
Function TEditDo.GetDuplicates:TDuplicates;
begin
   result:=FValueList.Duplicates;
end;
Procedure TEditDo.SetSorted(Value:Boolean);
begin
   if Value<>FValueList.Sorted then
      FValueList.Sorted:=Value;
end;
Function TEditDo.GetSorted:Boolean;
begin
   result:=FValueList.Sorted;
end;
procedure TEditDo.WMKillFocus(var Message: TWMSetFocus);
begin
   inherited;
   if ftmp<>Text then
      FValueList.Add(Text);
end;
procedure TEditDo.WMSetFocus(var Message: TWMSetFocus);
begin
   inherited;
   ftmp:=Text;
end;
function TEditDo.GetValue:String;
begin
   result:=iif(FIndexValue<>-1,FValueList[FIndexValue],'');
end;
function TEditDo.GetListValues:TStringList;
begin
   result:=self.FValueList;
end;
procedure TEditDo.Add(Const Cad:string);
begin
   FValueList.Add(Cad);
end;
function TEditDo.Locate(cad:string):Boolean;
   var I:integer;
begin
   FLocType:=ltLocate;
   FValueList.FromIndex:=false;
   FCad:= Cad;
   result:=FValueList.Locate(FCad,I);
   if result then
      FIndexValue:=I
   else
      FLocType:=ltNone;
end;
function TEditDo.Contains(cad:string):Boolean;
   var I:integer;
begin
   FLocType:=ltContains;
   FValueList.FromIndex:=false;
   FCad:= Cad;
   result:=FValueList.Contains(FCad,I);
   if result then
      FIndexValue:=I
   else
      FLocType:=ltNone;
end;
function TEditDo.BeginWith(cad:string):Boolean;
   var I:integer;
begin
   FLocType:=ltBeginWith;
   FValueList.FromIndex:=false;
   FCad:= Cad;
   result:=FValueList.BeginWith(FCad,I);
   if result then
      FIndexValue:=I
   else
      FLocType:=ltNone;
end;
Function TEditDo.FindNext:Boolean;
   var i:integer;
begin
   result:=false;
   FValueList.FromIndex:=true;
   i:=FIndexValue;
   case FLocType of
      ltLocate:
        begin
           result:=FValueList.Locate(FCad,I);
           if result then
              FIndexValue:=I
           else
              FLocType:=ltNone;
        end;
      ltContains:
        begin
           result:=FValueList.Contains(FCad,I);
           if result then
              FIndexValue:=I
           else
              FLocType:=ltNone;
        end;
      ltBeginWith:
        begin
           result:=FValueList.BeginWith(FCad,I);
           if result then
              FIndexValue:=I
           else
              FLocType:=ltNone;
        end;
   else
   end;
end;
function TEditDo.Next:String;
begin
   result:=text;
   if (fvaluelist.Count>0) and (findexvalue<fvaluelist.count-1) then
   begin
     inc(findexvalue);
     result:=fvaluelist[findexvalue];
   end
   else
      raise Exception.Create(sNoMoreValuesErrMsg);
//   else
//      result:=fvaluelist[findexvalue];
end;
function TEditDo.Previous:String;
begin
   result:=text;
   if (fvaluelist.Count>0) and (findexvalue>0) then
   begin
     dec(findexvalue);
     result:=fvaluelist[findexvalue];
   end
   else
      raise Exception.Create(sNoMoreValuesErrMsg);
//   else
//      result:=fvaluelist[findexvalue];
end;
Procedure TEditDo.SetIndexValue(Value:Integer);
begin
   if (Value>-1) and (Value<fvaluelist.Count) then
      findexvalue:=Value
   else
      raise Exception.Create(sUnboundNoParameterErrMsg);
end;
procedure Register;
begin
  RegisterComponents('i-vcl EditB', [TEditDo]);
end;

end.
