unit EllipButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls,Graphics,funciones,forms;

type
  TFocusType=(ftColor,ftNormal);    // Fipo de Foco : Normal o cambio de color de fondo
  TButtonType=(btCaption,btImage);  // Tipo de boton : Texto o imagen
  TEllipButton = class(TCustomControl)
  private
    FDown:Boolean;             // Botn pulsado
    FColor:TColor;             // Color de fondo del botn
    FPicture:TBitmap;          // Imagen del botn
    FPictureDisabled:TBitmap;  // Imagen del botn inhabilitado
    FFocusType:TFocusType;     // tipo del foco
    FFocusColor:TColor;        // color del foco
    FButtonType:TButtonType;   // Tipo de botn
    procedure SetColor(Value:TColor);
    procedure SetPicture(Value:Tbitmap);
    procedure SetPictureDisabled(Value:Tbitmap);
    procedure SetButtonType(Value:TButtontype);
    { Private declarations }
  protected
    procedure Paint;override;
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure WMKeyUp(var Message: TWMKeyUp); message WM_KEYUP;
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged); message WM_WINDOWPOSCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    { Protected declarations }
  published
    property Color:TColor Read FColor write SetColor default clbtnFace;
    property Picture:TBitmap read FPicture write SetPicture;
    property PictureDisabled:TBitmap read FPictureDisabled write SetPictureDisabled;
    property FocusType:TFocusType read FFocusType write FFocusType default ftNormal;
    property FocusColor:TColor read FFocusColor write FFocusColor default clCream;
    property ButtonType : TButtonType read FButtonType write SetButtonType default btCaption;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Action;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Constraints;
    property Enabled;
    property Font;
    property ParentBiDiMode;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnClick;
    property OnContextPopup;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    { Published declarations }
  end;

procedure Register;

implementation
constructor TEllipButton.Create(AOwner: TComponent);
const
  BtnStyle = [csCaptureMouse,csSetCaption,
    csDesignInteractive, csClickEvents,csDisplayDragImage,
    csFixedWidth, csFixedHeight,
    csReplicatable,csReflector,csActionClient,csOpaque];
begin
  inherited Create(AOwner);
  FDown:=False;
  TabStop:=true;
  width:=25;
  height:=25;
  FColor:=clbtnFace;
  FPicture:=TBitmap.Create;
  FPictureDisabled:=TBitmap.Create;
  FFocusColor:=clCream;
  FFocusType:=ftNormal;
  FButtonType:=btCaption;
  controlstyle:= btnStyle;
end;
destructor TEllipButton.Destroy;
begin
  if assigned(FPicture) then
     FPicture.Free;
  if assigned(FPictureDisabled) then
     FPictureDisabled.Free;
  inherited Destroy;
end;
procedure TEllipButton.SetButtonType(Value:TButtonType);
begin
   if FButtonType<>Value then
   begin
      FButtonType:=Value;
      repaint;
   end;
end;
procedure TEllipButton.SetPicture(Value: TBitmap);
begin
  FPicture.Assign(Value);
  fpicture.TransparentColor:=fpicture.Canvas.Pixels[1,1];
  if enabled then
     repaint;
end;
procedure TEllipButton.SetPictureDisabled(Value: TBitmap);
begin
  FPictureDisabled.Assign(Value);
  fpictureDisabled.TransparentColor:=fpictureDisabled.Canvas.Pixels[1,1];
  if not enabled then
     repaint;
end;
procedure TEllipButton.SetColor(Value:TColor);
begin
   if FColor<>value then
   begin;
      fColor:=Value;
      repaint;
   end;
end;
procedure TEllipButton.CMTextChanged(var Message: TMessage);
begin
   inherited;
   repaint;
end;
procedure TEllipButton.CMEnabledChanged(var Message: TMessage);
begin
   inherited;
   repaint;
end;
procedure TEllipButton.WMSetFocus(var Message: TWMSetFocus);
begin
   inherited;
   repaint;
end;
procedure TEllipButton.WMKillFocus(var Message: TWMSetFocus);
begin
   inherited;
   FDown:=false;
   repaint;
end;
procedure TEllipButton.WMLButtonDown(var Message: TWMLButtonDown);
begin
   inherited;
   FDown:=True;
   setfocus;
   repaint;
end;
procedure TEllipButton.WMLButtonUp(var Message: TWMLButtonUp);
begin
   inherited;
   FDown:=false;
   repaint;
end;
procedure TEllipButton.WMKeyDown(var Message: TWMKeyDown);
begin
   inherited;
   if Char(message.CharCode)=' ' then
      if not Fdown then
      begin
         FDown:=True;
         repaint;
         Click;
      end;
end;
procedure TEllipButton.WMKeyUp(var Message: TWMKeyUp);
begin
   inherited;
   if fdown then
   begin
      FDown:=False;
      repaint;
   end;
end;
procedure TEllipButton.CMMouseLeave(var Message: TMessage);
begin
   inherited;
   if FDown then
   begin
      Fdown:=false;
      repaint;
   end;
end;
procedure TEllipButton.WMWindowPosChanged(var Message: TWMWindowPosChanged);
var r:TRect;
    newShape:HRGN;
begin
   inherited;
   r:=clientrect;
   NewShape:=CreateEllipticRgn(r.Left,r.Top,r.Right,r.Bottom);
   SetWindowRgn(Handle,newshape,true);
   deleteobject(newshape);
end;
procedure TEllipButton.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if  (CharCode = VK_RETURN) and focused and
      (KeyDataToShiftState(Message.KeyData) = [])  then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TEllipButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if (IsAccel(CharCode, Caption) and CanFocus)  then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TEllipButton.Paint;
var r,rb:TRect;
   procedure DrawButtonImage;
   begin
      with self.Canvas do
      begin
         rb.Left:=r.Left+(((r.Right-r.Left) div 2)-(fpicture.Width div 2));
         rb.Top:=r.Top+(((r.Bottom-r.Top) div 2)-(fpicture.Height div 2));
         fpictureDisabled.Transparent:=true;
         fpicture.Transparent:=true;
         if not enabled and not fpicturedisabled.Empty then
            canvas.Draw(iif(Fdown,rb.Left+1,rb.Left),iif(FDown,rb.Top+1,rb.Top),fpictureDisabled)
         else
            canvas.Draw(iif(Fdown,rb.Left+1,rb.Left),iif(FDown,rb.Top+1,rb.Top),fpicture);
      end;
   end;
   procedure DrawButtonText;
   begin
      with self.Canvas do
      begin
         setbkmode(Handle,TRANSPARENT);
         canvas.Font:=self.Font;
         rb:=r;
         if Fdown then
            incpoint(rb.topleft,1);
         if not enabled then
         begin
            incpoint(rb.topleft,2);
            canvas.Font.Color:=clHighlightText;
            drawtext(Handle,pchar(caption),length(caption),rb,DT_CENTER+DT_SINGLELINE+DT_VCENTER);
            decpoint(rb.topleft,2);
            canvas.Font.Color:=clGrayText;
         end;
         drawtext(Handle,pchar(caption),length(caption),rb,DT_CENTER+DT_SINGLELINE+DT_VCENTER);
      end;
   end;
begin
   with canvas do
   begin
      r:=clientrect;
      IncPoint(r.TopLeft);
      DecPoint(r.BottomRight,2);
      Brush.Style:=bsSolid;
      Pen.Width:=1;
      Brush.Color:=iif(ffocustype=ftColor,iif(focused,FFocusColor,FColor),FColor);
      Pen.Color:=clbtnFace;
      Ellipse(r.Left+1,r.Top+1,r.Right-1,r.Bottom-1);
      if FButtonType=btImage then
         drawButtonImage
      else
         drawButtonText;
      pen.Width:=1;
      if not FDown then
      begin
         pen.Width:=2;
         Pen.Color:=clbtnShadow;
         Arc(r.Left+1,r.Top+1,r.Right-1,r.Bottom-1,r.Left,r.Bottom,r.Right,r.Top);
         pen.Width:=1;
         Pen.Color:=cl3DDkShadow;
         Arc(r.Left,r.Top,r.Right,r.Bottom,r.Left,r.Bottom,r.Right,r.Top);
         pen.Width:=2;
         Pen.Color:=cl3DLight;
         Arc(r.Left+1,r.Top+1,r.Right-1,r.Bottom-1,r.Right,r.Top,r.Left,r.Bottom);
         pen.Width:=1;
         Pen.Color:=clBtnHighlight;
         Arc(r.Left,r.Top,r.Right,r.Bottom,r.Right,r.Top,r.Left,r.Bottom);
      end
      else
      begin
         Pen.Color:= cl3DLight;
         Arc(r.Left+1,r.Top+1,r.Right-1,r.Bottom-1,r.Left,r.Bottom,r.Right,r.Top);
         Pen.Color:=clbtnShadow;
         Arc(r.Left+1,r.Top+1,r.Right-1,r.Bottom-1,r.Right,r.Top,r.Left,r.Bottom);
      end;
      pen.Width:=1;
      Pen.Color:=clblack;
      Brush.Style:=bsclear;
      if Focused then
      begin
         if (FFocusType=ftNormal) then
         begin
            pen.Mode:=pmNot;
            Pen.Style:=psDot;
            IncPoint(r.TopLeft,5);
            DecPoint(r.BottomRight,5);
            Ellipse(r);
            pen.Mode:=pmCopy;
         end;
         Pen.Style:=psSolid;
         r:=clientrect;
         IncPoint(r.TopLeft);
         DecPoint(r.BottomRight,2);
         Ellipse(r);
      end;
      Pen.Style:=psSolid;
   end;
end;

procedure Register;
begin
  RegisterComponents('i-griegavcl', [TEllipButton]);
end;

end.
