unit FillLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,graphics,dialogs;
type
  TAngles=(A0,A90,A180,A270);

  TFillLabel = class(TCustomLabel)
  private
    FPen:TPen;
    FBrush:TBrush;
    FPattern:TBitmap;
    FAngle:TAngles;
    FFilled:Boolean;
    procedure change(Sender:TObject);
    procedure SetPattern(Value:TBitmap);
    procedure SetAngle(Value:TAngles);
    Procedure SetFilled(Value:Boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    { Private declarations }
  protected
     procedure Paint; override;
    { Protected declarations }
  public
     constructor Create(AOwner:TComponent); override;
     Destructor Destroy;override;
    { Public declarations }
  published
    property Pen:TPen read FPen write FPen;
    property Brush:TBrush read FBrush write FBrush;
    Property Pattern:TBitmap read FPattern write SetPattern;
    Property Angle:TAngles read FAngle write SetAngle default A0;
    Property Filled:Boolean read FFilled write SetFilled default true;
    property Align;
    property Anchors;
    property AutoSize;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
    { Published declarations }
  end;
const
  AnglesValues:Array[A0..A270] of Integer=(0,90,180,270);

procedure Register;

implementation
procedure TFillLabel.CMFontChanged(var Message: TMessage);
var
   Metrics:TTextMetric;
begin
   inherited;
   if parent<>nil then
   begin
      gettextmetrics(canvas.Handle,metrics);
      if not((Metrics.tmPitchAndFamily and TMPF_TRUETYPE) =TMPF_TRUETYPE) and filled then
      begin
         showmessage('Slo fuentes truetype');
         self.Font.Name:='Arial Black';
      end
   end;
end;

procedure TFillLabel.SetFilled(Value:Boolean);
begin
   if FFilled<>Value then
   begin
      FFilled:=Value;
      invalidate;
   end;
end;
procedure TFillLabel.SetAngle(Value:TAngles);
begin
  if (Value<>FAngle) then
  begin
     FAngle:=Value;
     Invalidate;
  end;
end;
procedure TFillLabel.Setpattern(Value:TBitmap);
begin
   Fpattern.assign(Value);
   invalidate;
end;
procedure TFillLabel.change;
begin
   invalidate;
end;
constructor TFillLabel.Create(AOwner:TComponent);
begin
   inherited;
   FPen:=TPen.create;
   FBrush:=TBrush.Create;
   FPen.OnChange:=change;
   FBrush.OnChange:=Change;
   Fpattern:=TBitmap.Create;
   Font.Name:='Arial Black';
   Font.Size:=30;
   Layout:=tlCenter;
   Autosize:=true;
   FFilled:=True;
end;
Destructor TFillLabel.Destroy;
begin
   FPen.Free;
   FBrush.Free;
   if assigned(Fpattern) then
      Fpattern.Free;
   inherited;
end;
procedure TFillLabel.Paint;
   procedure DoDrawText(var Rect: TRect);
   var
      logfont:TLogFont;
      x,y:Integer;
   begin
       x:=0;
       y:=0;
       Canvas.Font := Font;
       case FAngle of
          A0:begin
                X:=0;
                y:=0;
                if autosize then
                   if Align=alNone then
                   begin
                     self.Width:=canvas.TextWidth(Caption)+1;
                     self.Height:=canvas.TextHeight('O');
                   end;
             end;
          A180:begin
                X:=rect.Right;
                y:=canvas.TextHeight('O');
                if autosize then
                   if Align=alNone then
                   begin
                      self.Width:=canvas.TextWidth(Caption)+1;
                      self.Height:=canvas.TextHeight('O');
                   end;
                end;
          A270:begin
                X:=canvas.TextHeight('O');
                y:=0;
                if autosize then
                   if Align=alNone then
                   begin
                     self.Height:=canvas.TextWidth(Caption)+1;
                     self.Width:=canvas.TextHeight('O');
                   end;
                end;
          A90:begin
                X:=0;
                y:=rect.Bottom;
                if autosize then
                   if Align=alNone then
                   begin
                     self.Height:=canvas.TextWidth(Caption)+1;
                     self.Width:=canvas.TextHeight('O');
                   end;
                end;
       end;
       getobject(canvas.Font.Handle,sizeof(LogFont),@Logfont);
       logfont.lfEscapement:=AnglesValues[FAngle]*10;
       logfont.lfOrientation:=AnglesValues[FAngle]*10;
       logfont.lfOutPrecision := OUT_TT_ONLY_PRECIS;
       canvas.Font.Handle:=createfontIndirect(logfont);
       canvas.Textout(X,Y,Caption);
//       DrawText(Canvas.Handle, PChar(caption), Length(caption), Rect, Flags);
   end;

var
  Rect: TRect;

begin
  with Canvas do
  begin
    if FFilled then
       Beginpath(Handle);
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    DoDrawText(Rect);
    if FFilled then
       EndPath(Handle);
    Pen.Assign(FPen);
    Brush.Assign(FBrush);
    try
       if not Fpattern.Empty and (Brush.Style=bsSolid) then
          Brush.Bitmap:=Fpattern;
    finally
       if FFilled then
          StrokeAndFillPath(Handle);
       Brush.Bitmap:=nil;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('i-vcl Labels', [TFillLabel]);
end;

end.
