unit GradLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,graphics;

type
  TDirection = (dHorizontal, dVertical,dCenter,dTopLeftCorner,dTopRightCorner,dBottomRightCorner,dBottomLeftCorner);  {Tipo de direccin del gradiente}
  TGradLabel = class(TCustomLabel)
  private
    FDirection : TDirection;   {Direccin del gradiente}
    FFromColor, FToColor : TColor;     {Color del gradiente}
    FPercent:Integer;
    procedure SetDirection(valor : TDirection);
    procedure SetFromColor(valor : TColor);
    procedure SetToColor(valor : TColor);
    procedure SetPercent(Value:Integer);
    { Private declarations }
  protected
    procedure Paint; override;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    { Public declarations }
  published
    property Direction : TDirection read FDirection write SetDirection default dVertical;
    property FromColor : TColor read FFromColor write SetFromColor default clBlue;
    property ToColor : TColor read FToColor write SetToColor default clBlack;
    property GradPercent:Integer read FPercent write SetPercent;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
    { Published declarations }
  end;

procedure Register;

implementation
procedure TGradLabel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result:= -1;
end;
constructor TGradLabel.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FDirection:=dVertical;
  FFromColor:=clWhite;
  FToColor:=clBtnFace;
  Width:=100;
  Transparent:=TRUE;
  FPercent:=75;
end;

procedure TGradLabel.SetDirection(Valor : TDirection);
begin
  if FDirection <> valor then
  begin
    FDirection := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;
procedure TGradLabel.SetPercent(Value:Integer);
begin
   If FPercent<>value then
   begin
      FPercent:=value;
      Repaint;
   end;
end;
procedure TGradLabel.SetFromColor(Valor : TColor);
begin
  if FFromColor <> Valor then
  begin
    FFromColor := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;

procedure TGradLabel.SetToColor(Valor : TColor);
begin
  if FToColor <> Valor then
  begin
    FToColor := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;

procedure TGradLabel.Paint;
var
  RGBDesde, RGBHasta, RGBDif : array[0..2] of byte;  {Colores inicial y final y diferencia de colores}
  contador, Rojo, Verde, Azul : integer;
  Banda : TRect;                                     {Coordenadas del recuadro a pintar}
  Factor : array[0..2] of shortint;                  {+1 si gradiente creciente o -1 en caso decreciente}
  Percent:integer;
//  C:TCanvas;
//  DC:HDC;
begin
//  DC:=self.Canvas.Handle;
//  c:=tcanvas.Create;
//  c.Handle:=dc;
  with canvas do
  begin
  percent:=trunc(Width*(FPercent/100));
  RGBDesde[0]:=GetRValue(ColorToRGB(FFromColor));
  RGBDesde[1]:=GetGValue(ColorToRGB(FFromColor));
  RGBDesde[2]:=GetBValue(ColorToRGB(FFromColor));    {Se descomponen los colores en rojo, verde y azul}
  RGBHasta[0]:=GetRValue(ColorToRGB(FToColor));
  RGBHasta[1]:=GetGValue(ColorToRGB(FToColor));
  RGBHasta[2]:=GetBValue(ColorToRGB(FToColor));
  for contador:=0 to 2 do         {Calculo de RGBDif[] y factor[]}
  begin
    RGBDif[contador]:=Abs(RGBHasta[contador]-RGBDesde[contador]);
    If RGBHasta[contador]>RGBDesde[contador] then factor[contador]:=1 else factor[contador]:=-1;
  end;
  Canvas.Pen.Style:=psSolid;            {Asignamos el estilo del pen}
  Canvas.Pen.Mode:=pmCopy;              {Idem modo}
  if FDirection = dHorizontal then      {Si el canvas es horizontal...}
  begin
    canvas.copymode:=cmSrcPaint;
    Banda.Left:=0;                      {Coordenadas recuadro}
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,height,256);
      Banda.Bottom:=MulDIv(contador+1,height,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);           {Pintamos el recuadro}
    end;
  end;
  if FDirection = dVertical then         {Gradiente vertical}
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    for contador:=0 to 255 do
    begin
      Banda.Left:=MulDiv(contador,Percent,256);
      Banda.Right:=MulDIv(contador+1,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dCenter then         {Gradiente centro}
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,Banda.bottom,256);
      Banda.Bottom:=Height-MulDIv(contador,Banda.bottom,256);
      Banda.Left:=MulDiv(contador,Banda.Right,256);
      Banda.Right:=Percent-MulDIv(contador,Banda.Right,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;

  if FDirection = dBottomRightCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Bottom:=Height-MulDIv(contador,hEIGHT,256);
      Banda.Right:=Percent-MulDIv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dTopLeftCorner then
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,HEIGHT,256);
      Banda.Left:=MulDiv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dBottomLeftCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Bottom:=Height-MulDIv(contador,HEIGHT,256);
      Banda.Left:=MulDiv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dTopRightCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,HEIGHT,256);
      Banda.Right:=Percent-MulDIv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  bitblt(Handle,0,0,self.Width,self.Height,Handle,0,0,srccopy);
  end;
  inherited paint;
end;
procedure Register;
begin
  RegisterComponents('i-vcl Labels', [TGradLabel]);
end;

end.
