unit GradLabel3D;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,graphics,gradlabel, Label3D;

type
  TGradLabel3D = class(TLabel3D)
  private
    FDirection : TDirection;   {Direccin del gradiente}
    FFromColor, FToColor : TColor;     {Color del gradiente}
    FPercent:Integer;
    procedure SetDirection(valor : TDirection);
    procedure SetFromColor(valor : TColor);
    procedure SetToColor(valor : TColor);
    procedure SetPercent(Value:Integer);
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    { Public declarations }
  published
     property Direction : TDirection read FDirection write SetDirection default dVertical;
    property FromColor : TColor read FFromColor write SetFromColor default clBlue;
    property ToColor : TColor read FToColor write SetToColor default clBlack;
    property GradPercent:Integer read FPercent write SetPercent;

    { Published declarations }
  end;

procedure Register;

implementation
constructor TGradLabel3D.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FDirection:=dVertical;
  FFromColor:=clWhite;
  FToColor:=clBtnFace;
  Width:=100;
  Transparent:=TRUE;
  FPercent:=75;
end;

procedure TGradLabel3D.SetDirection(Valor : TDirection);
begin
  if FDirection <> valor then
  begin
    FDirection := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;
procedure TGradLabel3D.SetPercent(Value:Integer);
begin
   If FPercent<>value then
   begin
      FPercent:=value;
      Repaint;
   end;
end;
procedure TGradLabel3D.SetFromColor(Valor : TColor);
begin
  if FFromColor <> Valor then
  begin
    FFromColor := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;

procedure TGradLabel3D.SetToColor(Valor : TColor);
begin
  if FToColor <> Valor then
  begin
    FToColor := Valor;
    Repaint;              {Fuerza redibujado del gradiente}
  end;
end;

procedure TGradLabel3D.Paint;
var
  RGBDesde, RGBHasta, RGBDif : array[0..2] of byte;  {Colores inicial y final y diferencia de colores}
  contador, Rojo, Verde, Azul : integer;
  Banda : TRect;                                     {Coordenadas del recuadro a pintar}
  Factor : array[0..2] of shortint;                  {+1 si gradiente creciente o -1 en caso decreciente}
  Percent:integer;
begin
  percent:=trunc(Width*(FPercent/100));
  RGBDesde[0]:=GetRValue(ColorToRGB(FFromColor));
  RGBDesde[1]:=GetGValue(ColorToRGB(FFromColor));
  RGBDesde[2]:=GetBValue(ColorToRGB(FFromColor));    {Se descomponen los colores en rojo, verde y azul}
  RGBHasta[0]:=GetRValue(ColorToRGB(FToColor));
  RGBHasta[1]:=GetGValue(ColorToRGB(FToColor));
  RGBHasta[2]:=GetBValue(ColorToRGB(FToColor));
  for contador:=0 to 2 do         {Calculo de RGBDif[] y factor[]}
  begin
    RGBDif[contador]:=Abs(RGBHasta[contador]-RGBDesde[contador]);
    If RGBHasta[contador]>RGBDesde[contador] then factor[contador]:=1 else factor[contador]:=-1;
  end;
  Canvas.Pen.Style:=psSolid;            {Asignamos el estilo del pen}
  Canvas.Pen.Mode:=pmCopy;              {Idem modo}
  if FDirection = dHorizontal then      {Si el canvas es horizontal...}
  begin
    canvas.copymode:=cmSrcPaint;
    Banda.Left:=0;                      {Coordenadas recuadro}
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,height,256);
      Banda.Bottom:=MulDIv(contador+1,height,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);           {Pintamos el recuadro}
    end;
  end;
  if FDirection = dVertical then         {Gradiente vertical}
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    for contador:=0 to 255 do
    begin
      Banda.Left:=MulDiv(contador,Percent,256);
      Banda.Right:=MulDIv(contador+1,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dCenter then         {Gradiente centro}
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,Banda.bottom,256);
      Banda.Bottom:=Height-MulDIv(contador,Banda.bottom,256);
      Banda.Left:=MulDiv(contador,Banda.Right,256);
      Banda.Right:=Percent-MulDIv(contador,Banda.Right,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;

  if FDirection = dBottomRightCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Bottom:=Height-MulDIv(contador,hEIGHT,256);
      Banda.Right:=Percent-MulDIv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dTopLeftCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,HEIGHT,256);
      Banda.Left:=MulDiv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dBottomLeftCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Bottom:=Height-MulDIv(contador,HEIGHT,256);
      Banda.Left:=MulDiv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  if FDirection = dTopRightCorner then         
  begin
    Banda.Top:=0;
    Banda.Bottom:=Height;
    Banda.left:=0;
    Banda.Right:=Percent;
    for contador:=0 to 255 do
    begin
      Banda.Top:=MulDiv(contador,HEIGHT,256);
      Banda.Right:=Percent-MulDIv(contador,Percent,256);
      Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
      Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
      Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
      Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
      Canvas.FillRect(Banda);
    end;
  end;
  inherited paint;
end;
procedure Register;
begin
  RegisterComponents('i-vcl Labels', [TGradLabel3D]);
end;

end.
