unit GradienteM;

interface

uses
  windows,dialogs,Classes, Controls, math,Graphics,ExtCtrls, WinTypes, WinProcs,messages,sysutils,igriegaclasses;
type
  TColorItem=class(TCollectionItem)
     private
        FColor:TColor;
        procedure SetColor(value:TColor);
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Color:TColor read FColor Write SetColor;
  end;
  TColors=Class(TCollection)
     private
        FOwner:TPersistent;
        function GetItem(Index:Integer):TColorItem;
        Procedure SetItem(Index:Integer;Value:TColorItem);
     protected
        function GetOwner:TPersistent;override;
        procedure Notify(Item: TCollectionItem; Action: TCollectionNotification);override;
     public
         Constructor Create(Owner:TPersistent);
         Function Add:TColorItem;
         property Items[Index:Integer]:TColorItem read Getitem write SetItem; default;

     published
  end;
  TGradienteM = class(TGraphicControl)
  private
    FColors:TColors;
    FDireccion : TDirection;   {Direccin del gradiente}
    FColorDesde, FColorHasta : TColor;     {Color del gradiente}
    FBevel:TBevelCut;
    FBevelWidth:Integer;
    FBevelOuter:TBevelCut;
    FBevelOuterWidth:Integer;
    FBevelInner:Integer;
    procedure SetDireccion(valor : TDirection);
    procedure SetBevel(value:TBevelCut);
    procedure SetBevelWidth(value:integer);
    procedure SetBevelInner(value:integer);
    procedure SetBevelOuter(value:TBevelCut);
    procedure SetBevelOuterWidth(value:integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner : TComponent); override;
    Destructor Destroy;override;
  published
    property Direccion : TDirection read FDireccion write SetDireccion default dVertical;
    property Align;     {Redeclaracin de la propiedad como publicada}
    property Colors:TColors read FColors write FColors;
    property Bevel:TBevelCut read FBevel write SetBevel default bvNone;
    property BevelWidth:Integer read FBevelWidth write SetBevelWidth default 1;
    property BevelOuter:TBevelCut read FBevelOuter write SetBevelOuter default bvNone;
    property BevelOuterWidth:Integer read FBevelOuterWidth write SetBevelOuterWidth default 1;
    property BevelInner:Integer read FBevelInner write SetBevelInner default 3;
  end;
implementation
constructor TColorItem.Create;
begin
   inherited;
   SetColor(clbtnFace);
end;
Procedure TColorItem.SetColor(value:TColor);
begin
  if fcolor<>value then
  begin
     FColor := value;
     if TColors(GetOwner).Owner.ClassName='TGradienteM' then
        TGradienteM(TColors(GetOwner).Owner).Invalidate;
  end;
end;
function TColorItem.GetDisplayName: string;
begin
   result:='Color'+inttostr(index);
end;
constructor TColors.Create(Owner:TPersistent);
begin
   inherited Create(TColorItem);
   Fowner:=owner;
   self.Add;
   self.Add;
   self[0].Color:=clActiveCaption;
   self[1].Color:=clGradientActiveCaption;
end;
procedure TColors.Notify(Item: TCollectionItem; Action: TCollectionNotification);
begin
  inherited;
  if ((Action=cnDeleting) or (Action=cnExtracting)) and not(csDestroying in TGradienteM(Owner).ComponentState)
      and (self.Owner.ClassName='TGradienteM') then
        TGradienteM(Owner).Invalidate;
end;
function TColors.GetItem(Index:Integer):TColorItem;
begin
   result:=TColorItem(inherited GetItem(Index));
end;
Procedure TColors.SetItem(Index:Integer;Value:TColorItem);
begin
   inherited SetItem(Index,Value);
end;
Function TColors.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TColors.Add:TColorItem;
begin
   result:=TColorItem(Inherited Add);
end;

constructor TGradienteM.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);       {Siempre lo primero a hacer}
  FDireccion:=dVertical;        {Valores por defecto}
  FColorDesde:=clBlue;
  FColorHasta:=clBtnFace;
  Width:=100;
  Height:=100;
  FBevel:=bvNone;
  FBevelWidth:=1;
  FBevelOuter:=bvNone;
  FBevelOuterWidth:=1;
  FBevelInner:=3;
  FColors:=TColors.Create(self);
end;
Destructor TGradienteM.Destroy;
begin
   fcolors.Free;
   inherited;
end;
procedure TGradienteM.SetBevel(value:TBevelCut);
begin
   if FBevel<>value then
   begin
      FBevel:=value;
      invalidate;
   end;
end;
procedure TGradienteM.SetBevelWidth(value:integer);
begin
   if value>0 then
   begin
      FBevelWidth:=value;
      invalidate;
   end;
end;
procedure TGradienteM.SetBevelOuter(value:TBevelCut);
begin
   if FBevelOuter<>value then
   begin
      FBevelOuter:=value;
      invalidate;
   end;
end;
procedure TGradienteM.SetBevelOuterWidth(value:integer);
begin
   if value>0 then
   begin
      FBevelOuterWidth:=value;
      invalidate;
   end;
end;
procedure TGradienteM.SetBevelInner(value:integer);
begin
   if value>=0 then
   begin
      FBevelInner:=value;
      invalidate;
   end;
end;
procedure TGradienteM.SetDireccion(Valor : TDirection);
begin
  if FDireccion <> valor then
  begin
    FDireccion := Valor;
    invalidate;
  end;
end;

procedure TGradienteM.Paint;
var
  RGBDesde, RGBHasta, RGBDif : array[0..2] of byte;  {Colores inicial y final y diferencia de colores}
  contador, Rojo, Verde, Azul : integer;
  Banda : TRect;                                     {Coordenadas del recuadro a pintar}
  Factor : array[0..2] of shortint;                  {+1 si gradiente creciente o -1 en caso decreciente}
  esp,i,ancho,alto:integer;
  rancho,ralto:integer;
begin
  with canvas do
  begin
  esp:=colors.Count-1;
  ancho:=width div esp;
  alto:=Height div esp;
  rancho:=0;
  ralto:=0;
  for i:=0 to esp-1 do
  begin
     if i=(esp-1) then
     begin
       rancho:=(width mod esp);
       ralto:=(Height mod esp);
     end;
     FColorDesde:=Colors[i].Color;
     FColorHasta:=Colors[i+1].Color;
     RGBDesde[0]:=GetRValue(ColorToRGB(FColorDesde));
     RGBDesde[1]:=GetGValue(ColorToRGB(FColorDesde));
     RGBDesde[2]:=GetBValue(ColorToRGB(FColorDesde));    {Se descomponen los colores en rojo, verde y azul}
     RGBHasta[0]:=GetRValue(ColorToRGB(FColorHasta));
     RGBHasta[1]:=GetGValue(ColorToRGB(FColorHasta));
     RGBHasta[2]:=GetBValue(ColorToRGB(FColorHasta));
     for contador:=0 to 2 do         {Calculo de RGBDif[] y factor[]}
     begin
       RGBDif[contador]:=Abs(RGBHasta[contador]-RGBDesde[contador]);
       If RGBHasta[contador]>RGBDesde[contador] then factor[contador]:=1 else factor[contador]:=-1;
     end;
     Canvas.Pen.Style:=psSolid;            {Asignamos el estilo del pen}
     Canvas.Pen.Mode:=pmCopy;              {Idem modo}
     if FDireccion = dHorizontal then      {Si el canvas es horizontal...}
     begin
       canvas.copymode:=cmSrcPaint;
       Banda.Left:=0;                      {Coordenadas recuadro}
       Banda.Right:=Width;
       for contador:=0 to 255 do
       begin
         Banda.Top:=(Alto*i)+MulDiv(contador,alto+ralto,256);
         Banda.Bottom:=(Alto*i)+MulDIv(contador+1,alto+ralto,256);
         Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
         Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
         Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
         Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
         Canvas.FillRect(Banda);           {Pintamos el recuadro}
       end;
     end;
     if FDireccion = dVertical then         {Gradiente vertical}
     begin
       Banda.Top:=0;
       Banda.Bottom:=Height;
       for contador:=0 to 255 do
       begin
         Banda.Left:=(ancho*i)+MulDiv(contador,ancho+rancho,256);
         Banda.Right:=(ancho*i)+MulDIv(contador+1,ancho+rancho,256);
         Rojo:=RGBDesde[0]+factor[0]*MulDiv(contador,RGBDif[0],255);
         Verde:=RGBDesde[1]+factor[1]*MulDiv(contador,RGBDif[1],255);
         Azul:=RGBDesde[2]+factor[2]*MulDiv(contador,RGBDif[2],255);
         Canvas.Brush.Color:=RGB(Rojo,Verde,Azul);
         Canvas.FillRect(Banda);
       end;
     end;
  end;
  banda:=clientrect;
  case FBevelOuter of
     bvRaised,bvSpace :frame3d(canvas,banda,clbtnhighlight,clbtnshadow,BevelOuterWidth );
     bvLowered :frame3d(canvas,banda,clbtnshadow,clbtnhighlight,BevelOuterWidth );
  end;
  banda:=bounds(banda.Left+FBevelInner,banda.Top+FBevelInner,
                width-((FBevelInner+BevelOuterWidth)*2),
                height-((FBevelInner+BevelOuterWidth)*2));
  case FBevel of
     bvRaised,bvSpace :frame3d(canvas,banda,clbtnhighlight,clbtnshadow,BevelWidth );
     bvLowered :frame3d(canvas,banda,clbtnshadow,clbtnhighlight,BevelWidth );
  end ;
  end;
end;

end.

