 unit HintComponent;

interface

uses
  Windows, Messages, SysUtils, Classes,graphics,controls,
  forms,dialogs,igriegastrs,funciones,math;

type
  THintBevel=(hbRaised,hbLowered,hbNone,hbEdgeBump,hbEdgeEtChed);
  TActivateHintEvent=procedure(Sender:TObject;var AHint:String;PHintControl:TControl) of object;
  THintComponent = class(TComponent)
  private
     FColor:TColor;
     FFont:TFont;
     FMaxSize:Integer;
     FActive:Boolean;
     FPicture:TPicture;
     FBevel:THintBevel;
     FBorderWidth:TBorderWidth;
     FHintEvent:TActivateHintEvent;
     procedure SetPicture(Value:TPicture);
     Procedure SetFont(Value:TFont);
  protected
    { Protected declarations }
  public
     Constructor Create(AOwner:TComponent);override;
     Destructor Destroy;override;
     procedure Activate;
     procedure DeActivate;
     property BorderWidth:TBorderWidth read FBorderWidth write fBorderWidth default 1;     
  published
    property Color:TColor read FColor Write FColor default clInfoBk;
    property Font:TFont read FFont write SetFont;
    Property MaxSize:Integer read FMaxSize write FMaxSize default -1;
    property Bevel:THintBevel read FBevel write FBevel default hbNone;
    property Picture:TPicture read FPicture Write SetPicture;
    property OnActivateHint:TActivateHintEvent read FHintEvent write FHintEvent;
  end;

  THintWindowigv = class(THintWindow)
  private
     FHintComponent:tHintComponent;
     FHintEvent:TActivateHintEvent;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Paint; override;
    procedure NCPaint(DC: HDC); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;override;
    function CalcHintRect(MaxWidth: Integer; const AHint: string;
      AData: Pointer): TRect; override;
    procedure ActivateHint(Rect: TRect; const AHint: string); override;
    property OnActivateHint:TActivateHintEvent read FHintEvent write FHintEvent;
    property BiDiMode;
    property Caption;
    property Color;
    property Font;
  end;

procedure Register;

implementation

function GetHintControl(Control: TControl): TControl;
begin
  Result := Control;
  while (Result <> nil) and not Result.ShowHint do Result := Result.Parent;
  if (Result <> nil) and (csDesigning in Result.ComponentState) then Result := nil;
end;


Constructor THintComponent.Create(AOwner:TComponent);
var i:integer;
begin
   for i:=0 to TForm(AOwner).ComponentCount-1 do
      if TForm(AOwner).Components[i].ClassType=THintComponent then
      begin
         raise Exception.Create(sMsgHintComponentExist);
      end;
   inherited;
   FPicture:=tpicture.Create;
   FActive:=False;
   FFont:=TFont.create;
   FColor:= clInfoBk;
   FMaxSize:=-1;
   FBevel:=hbNone;
   FBorderWidth:=1;
end;
Destructor THintComponent.Destroy;
begin
   FFont.Free;
   FPicture.Free;
   inherited;
end;
procedure THintComponent.SetFont(Value:TFont);
begin
   FFont.Assign(Value);
end;
procedure THintComponent.SetPicture(Value:TPicture);
begin
   FPicture.Assign(Value);
end;
procedure THintComponent.Activate;
begin
   if not FActive then
   begin
      hintwindowclass:=tHintWindowigv;
      FActive:=True;
   end;
end;
procedure THintComponent.DeActivate;
Begin
   if FActive then
   begin
      hintwindowclass:=tHintWindow;
      FActive:=False;
   end;
End;

constructor THintWindowigv.Create(AOwner: TComponent);
VAR
   i:integer;
begin
  inherited Create(AOwner);
  Color := $80FFFF;
  Canvas.Font := Screen.HintFont;
  Canvas.Brush.Style := bsClear;
  for i:=0 to screen.ActiveForm.ComponentCount-1  do
  begin
     if screen.ActiveForm.components[i].ClassType=THintComponent then
     begin
        FHintComponent:=tHintComponent(screen.ActiveForm.components[i]);
        Color := fHintComponent.Color;;
        Canvas.Font := fHintComponent.Font;
        Canvas.Brush.Style := bsClear;
        Canvas.Brush.Color:=fHintComponent.Color;
        if assigned(FHintComponent.OnActivateHint) then
           FHintEvent:=FHintComponent.OnActivateHint;
        BorderWidth:=fHintComponent.BorderWidth;
        break;
     end;
  end;
end;
destructor THintWindowigv.Destroy;
begin
   FHintEvent:=nil;
   fHintComponent:=nil;
   inherited;
end;
procedure THintWindowigv.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style :=WS_POPUP or WS_BORDER;
    WindowClass.Style := WindowClass.Style or CS_SAVEBITS;
    if NewStyleControls then ExStyle := WS_EX_TOOLWINDOW;
    AddBiDiModeExStyle(ExStyle);
  end;
end;

procedure THintWindowigv.Paint;
var
  R: TRect;

begin
  R := ClientRect;
  Canvas.Brush.Style := bsSolid;
  canvas.Brush.Color:=FHintComponent.Color;
  canvas.fillrect(R);
  Canvas.Brush.Style := bsClear;
  if assigned(FHintComponent.Picture.Graphic) then
     Inc(R.Left, FHintComponent.Picture.Graphic.Width+6)
  else
     Inc(R.Left, 2);
  Inc(R.Top, 2);
  DrawText(Canvas.Handle, PChar(Caption), -1, R, DT_LEFT or DT_NOPREFIX or
    DT_WORDBREAK or DrawTextBiDiModeFlagsReadingOnly);
  if assigned(FHintComponent.Picture.Graphic) then
  begin
     FHintComponent.fpicture.Graphic.Transparent:=true;
     if FHintComponent.FPicture.Graphic.ClassName='TBitmap' then
         FHintComponent.FPicture.Bitmap.TransparentMode:=tmAuto;
     canvas.Draw(2,2,FHintComponent.Picture.Graphic);
  end;
end;
function THintWindowigv.CalcHintRect(MaxWidth: Integer; const AHint: string; AData: Pointer): TRect;
var
   ms:integer;
begin
  ms:=0;
  if FHintComponent.MaxSize>0 then
    if assigned(FHintComponent.Picture.Graphic) then
       if FHintComponent.Picture.Graphic.Width>FHintComponent.MaxSize then
          ms:=50
       else
          ms:=FHintComponent.MaxSize-FHintComponent.Picture.Graphic.Width
    else
       ms:=FHintComponent.MaxSize;
  result:=ifthen((MaxWidth>ms) and (FHintComponent.MaxSize>0),Rect(0, 0, ms, 0),Rect(0, 0, MaxWidth, 0));
  DrawText(Canvas.Handle, PChar(AHint), -1, Result, DT_CALCRECT or DT_LEFT or
    DT_WORDBREAK or DT_NOPREFIX or DrawTextBiDiModeFlagsReadingOnly);
  if assigned(FHintComponent.Picture.Graphic) then
  begin
    Inc(Result.Right,FHintComponent.Picture.Graphic.Width+6);
    Result.Bottom:=max(result.Bottom-result.Top,FHintComponent.Picture.Graphic.Height)+2;
  end ;
  Inc(Result.Right,6);
  inc(Result.Bottom, 2);
end;
procedure THintWindowigv.NCPaint(DC: HDC);
var
  R: TRect;
begin
  R := Rect(0, 0, Width, Height);
  case fHintComponent.Bevel of
     hbRaised: Windows.DrawEdge(DC, R, EDGE_RAISED, BF_RECT);
     hbLowered: Windows.DrawEdge(DC, R, EDGE_SUNKEN, BF_RECT);
     hbEdgeBump: Windows.DrawEdge(DC, R, EDGE_BUMP, BF_RECT);
     hbEdgeEtChed: Windows.DrawEdge(DC, R, EDGE_ETCHED, BF_RECT);
  else
     BorderWidth:=0;
     Windows.Rectangle(DC,0,0,Width, Height);
  end;

end;
procedure THintWindowigv.ActivateHint(Rect: TRect; const AHint: string);
var vhint:string;
    R:TRect;
    w,h:integer;
begin
   vhint:=AHint;
   if assigned(FHintEvent) then
      FHintEvent(self,vhint,GetHintControl(FindDragTarget(mouse.CursorPos, True)));
   R:=CalcHintRect(Screen.Width,vhint,nil);
   w:=r.Right-r.Left;
   h:=r.Bottom-r.Top;
   R.TopLeft:=rect.TopLeft;
   r.Right:=rect.left+w;
   r.Bottom:=rect.Top+h;
   inherited ActivateHint(R,vhint);
end;
procedure Register;
begin
  RegisterComponents('i-vcl Non-Visual', [THintComponent]);
end;

end.
