unit HotSpotMapN;

interface

uses
  Windows, Messages, SysUtils, Classes,Graphics,ExtCtrls,controls,dialogs,menus;
const
  InkClassModevalues:array[0..11] of longint=(cmBlackness,cmDstInvert,cmMergeCopy,cmMergePaint,cmNotSrcCopy,
                 cmNotSrcErase,cmSrcAnd,cmSrcCopy,cmSrcErase,cmSrcInvert,cmSrcPaint,cmWhiteness);
type
  TInkClassMode=(imBlackness,imDstInvert,imMergeCopy,imMergePaint,imNotSrcCopy,
                 imNotSrcErase,imSrcAnd,imSrcCopy,imSrcErase,imSrcInvert,imSrcPaint,imWhiteness);
  TPPoint=class(TPersistent)
  private
     FX:Integer;
     FY:Integer;
  public
     Constructor Create;
  published
     property X:Integer read FX write FX;
     property Y:Integer read FY write FY;
  end;

  TPPointItem=class(TCollectionItem)
     private
        FPoint:TPPoint;
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Point:TPPoint read FPoint Write FPoint;
  end;

  TPoly=Class(TCollection)
     private
        FOwner:TPersistent;
        FHint:String;
        FHintColor:TColor;
        FPopupMenu:TPopupMenu;
        function GetItem(Index:Integer):TPPointItem;
        Procedure SetItem(Index:Integer;Value:TPPointItem);
     protected
        function GetOwner:TPersistent;override;
     public
        Constructor Create(Owner:TPersistent);
        Function Add:TPPointItem;
        property Items[Index:Integer]:TPPointItem read Getitem write SetItem; default;
     published
        property Hint:String read FHint write FHint;
        property HintColor:TColor read FHintColor write FHintColor default clInfoBk;
        property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu;
  end;

  TPolyItem=class(TCollectionItem)
     private
        FPoly:TPoly;
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Poly:TPoly read FPoly Write FPoly;
  end;

  TPolys=Class(TCollection)
     private
        FOwner:TPersistent;
        FLoadFrom:TFilename;
        function GetItem(Index:Integer):TPolyItem;
        Procedure SetItem(Index:Integer;Value:TPolyItem);
     protected
        function GetOwner:TPersistent;override;
     public
         Constructor Create(Owner:TPersistent);
         Function Add:TPolyItem;
         property Items[Index:Integer]:TPolyItem read Getitem write SetItem; default;
     published
  end;

  TOnHotClick=procedure(Sender : TObject;Polygon:integer) of object;
  TOnPolygonActive=TOnHotClick;
  THotSpotMapN = class(TGraphicControl)
  private
    FPolygons:TPolys;
    FOnHotClick:TOnHotClick;
    FOnPolygonActive:TOnPolygonActive;
    FHintW:THintWindow;
    FPolygon:integer;
    FPicture:TPicture;
    FStretch:Boolean;
    FProportional:Boolean;
    FCenter: Boolean;
    FTransparent:Boolean;
    FFramed:Boolean;
    FFrameSize:Integer;
    FInkColor:TColor;
    FInkGlass:Boolean;
    FInkGlassMode:TInkClassMode;
    FEventOutPolygon:Boolean;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure SetPicture(Value:TPicture);
    function GetPicture:TPicture;
    procedure SetPolygon(Value:Integer);
    procedure SetStretch(Value:Boolean);
    procedure SetProportional(Value:Boolean);
    procedure SetCenter(Value: Boolean);
    procedure SetTransparent(Value:Boolean);
    procedure SetFrameSize(Value:Integer);
    function DestRect: TRect;
    { Private declarations }
  protected
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);override;
     procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
     procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
     procedure Paint; override;
     function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;override;
    { Protected declarations }
  public
    constructor Create(AOwner:TComponent);override;
    Destructor Destroy;override;
    property Polygon:Integer read FPolygon write SetPolygon default -1;
    { Public declarations }
  published
    Property Polygons:TPolys read FPolygons write FPolygons;
    Property OnHotClick:TOnHotClick read FOnHotClick write FOnHotClick;
    Property Picture:TPicture read GetPicture write setpicture;
    property OnPolygonActive:TOnPolygonActive read FOnPolygonActive write FOnPolygonActive;
    property EventOutPolygon:Boolean read FEventOutPolygon write FEventOutPolygon default false;
    property ShowHint;
    property Stretch:Boolean read FStretch write SetStretch;
    property Proportional:Boolean read FProportional write SetProportional;
    property Center: Boolean read FCenter write SetCenter default False;
    property Transparent:Boolean read FTransparent write SetTransparent;
    property Framed:Boolean read FFramed write FFramed default false;
    property FrameSize:Integer read FFrameSize write SetFrameSize default 1;
    property InkColor:TColor read FInkColor write FInkColor default clRed;
    property InkGlass:Boolean read FInkGlass write FInkGlass default false;
    property InkGlassMode:TInkClassMode read FInkGlassMode write FInkGlassMode default imSrcCopy;
    property Align;
    property Anchors;
    property AutoSize;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;

    { Published declarations }
  end;

implementation
constructor THotSpotMapN.Create(AOwner:TComponent);
begin
   inherited;
   ControlStyle := ControlStyle + [csReplicatable]+[csOpaque];
   FPicture:=tpicture.Create;
   FPolygons:=TPolys.Create(self);
   FHintW:=THintWindow.Create(self);
   FPolygon:=-1;
   FEventOutPolygon:=false;
   FFramed:=false;
   FFrameSize:=1;
   FInkColor:=clRed;
   FInkGlass:=False;
   FInkGlassMode:=imSrcCopy;
end;
Destructor THotSpotMapN.Destroy;
begin
   Fpicture.Free;
   FPolygons.Free;
   FHintW.Free;
   inherited;
end;
procedure THotSpotMapN.SetFrameSize(Value:Integer);
begin
   if (FFrameSize<>Value) and (Value>0) then
      FFrameSize:=Value;
end;
function THotSpotMapN.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result := True;
  if not (csDesigning in ComponentState) or (Picture.Width > 0) and
    (Picture.Height > 0) then
  begin
    if Align in [alNone, alLeft, alRight] then
      NewWidth := Picture.Width;
    if Align in [alNone, alTop, alBottom] then
      NewHeight := Picture.Height;
  end;
end;
Procedure THotSpotMapN.SetProportional(Value:Boolean);
begin
   if FProportional<>Value then
   begin
      FProportional:=Value;
      repaint;
   end;
end;
procedure THotSpotMapN.SetTransparent(Value:Boolean);
begin
   if FTransparent<>Value then
   begin
      FTransparent:=Value;
      self.Picture.Graphic.Transparent:=Value;
      repaint;
   end;
end;
procedure THotSpotMapN.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
	FCenter := Value;
	repaint;
  end;
end;
procedure THotSpotMapN.SetStretch(Value:Boolean);
begin
   if FStretch<>value then
   begin
       FStretch:=Value;
       repaint;
   end;
end;
//********************************************************
//********************************************************
procedure THotSpotMapN.Paint;
var
   PolyPoints,PolyPointstmp:array[0..100] of TPoint;
   j:integer;
   nPoints:Integer;
   btmp:TBitmap;
   r:TRect;
   rgn:HRGN;
begin
   if csDesigning in ComponentState then
      with Canvas do
      begin
         Pen.Style := psDash;
         Brush.Style := bsClear;
         Rectangle(0, 0, Width, Height);
      end;

   if not Stretch then
      Canvas.Draw(0,0,self.Picture.Graphic)
   else
      canvas.StretchDraw(DestRect,self.Picture.Graphic);
   if not(csDesigning in ComponentState) then
   begin
      if (FPolyGon<>-1) and (framed or inkglass) then
      begin
         npoints:=self.Polygons.Items[FPolygon].Poly.Count;
         if npoints>100 then npoints:=100;
         for j:=0 to npoints-1 do
         begin
            PolyPoints[j].X:=self.Polygons.Items[FPolygon].Poly.Items[j].Point.X;
            PolyPoints[j].Y:=self.Polygons.Items[FPolygon].Poly.Items[j].Point.Y;
            PolyPointstmp[j].X:=PolyPoints[j].X+self.Left;
            PolyPointstmp[j].y:=PolyPoints[j].Y+self.Top;
         end;
         if InkGlass then
         begin
            btmp:=tbitmap.Create;
            rgn:=createpolygonrgn(polypointstmp,npoints,WINDING);
            getrgnbox(rgn,r);
            btmp.Width:=r.Right-r.Left;
            btmp.Height:=r.Bottom-r.Top;
            btmp.Canvas.Brush.Style:=bsSolid;
            btmp.Canvas.Brush.Color:=inkcolor;
            btmp.Canvas.FillRect(rect(0,0,btmp.Width,btmp.Height));
            selectcliprgn(Canvas.Handle,rgn);
            canvas.CopyMode:=InkClassModevalues[ord(InkGlassMode)];
            Canvas.CopyRect(clientrect,btmp.Canvas,rect(0,0,btmp.Width,btmp.Height));
            canvas.CopyMode:=cmSrcCopy;
            selectcliprgn(Canvas.Handle,0);
            deleteobject(rgn);
            btmp.Free;
         end;
         canvas.Brush.Style:=bsclear;
         if framed then
            self.canvas.Polygon(slice(polypoints,npoints));
      end;
   end;
end;
function THotSpotMapN.DestRect: TRect;
var
  w, h, cw, ch: Integer;
  xyaspect: Double;
begin
  w := Picture.Width;
  h := Picture.Height;
  cw := ClientWidth;
  ch := ClientHeight;
  if Stretch or (Proportional and ((w > cw) or (h > ch))) then
  begin
	if Proportional and (w > 0) and (h > 0) then
	begin
      xyaspect := w / h;
      if w > h then
      begin
        w := cw;
        h := Trunc(cw / xyaspect);
        if h > ch then  // woops, too big
        begin
          h := ch;
          w := Trunc(ch * xyaspect);
        end;
      end
      else
      begin
        h := ch;
        w := Trunc(ch * xyaspect);
        if w > cw then  // woops, too big
        begin
          w := cw;
          h := Trunc(cw / xyaspect);
        end;
      end;
    end
    else
    begin
      w := cw;
      h := ch;
    end;
  end;

  with Result do
  begin
    Left := 0;
    Top := 0;
    Right := w;
    Bottom := h;
  end;

  if Center then
	OffsetRect(Result, (cw - w) div 2, (ch - h) div 2);
end;
procedure THotSpotMapN.Notification(AComponent: TComponent;
      Operation: TOperation);
var
   i:integer;
begin
   inherited;
   if (AComponent.classtype=TPopupMenu) and (Operation=opRemove) then
   begin
      for i:=0 to self.polygons.count-1 do
         if self.Polygons[i].Poly.PopupMenu=AComponent then
            self.Polygons[i].Poly.PopupMenu:=nil;
   end;
end;

procedure THotSpotMapN.SetPolygon(Value:Integer);
begin
   if FPolygon<>value then
   begin
      FPolygon:=Value;
      repaint;
      if assigned(FOnPolygonActive) and (((value=-1) and EventOutPolygon) or (value<>-1)) then
         FOnPolygonActive(self,Value)
   end;
end;
procedure THotSpotMapN.SetPicture(Value:TPicture);
var
   tmp:TBitmap;
begin
   tmp:=tbitmap.Create;
   Tmp.Width:=value.Width;
   Tmp.Height:=value.Height;
   FPicture.Bitmap.Width:=value.Width;
   FPicture.Bitmap.Height:=value.Height;
   Tmp.Canvas.Draw(0,0,value.Graphic);
   FPicture.Assign(Tmp);
   tmp.Free;
   controlstyle:=controlstyle-[csopaque];
   Invalidate;
   controlstyle:=controlstyle+[csopaque];
end;
function THotSpotMapN.getpicture:TPicture;
begin
   result:=FPicture;
end;
procedure THotSpotMapN.CMMouseLeave(var Message: TMessage);
begin
  if fhintw.HandleAllocated then
     FHintw.ReleaseHandle;
  if Parent <> nil then
    Parent.Perform(CM_MOUSELEAVE, 0, Longint(Self));
  Polygon:=-1;
end;
procedure THotSpotMapN.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
var
   HotSpotRgn: HRGN;
   PolyPoints: array[0..100] of TPoint;
   i,j:integer;
   polyfound:boolean;
   npoints:integer;
   punto:tpoint;
begin
 if fhintw.HandleAllocated then
    FHintW.ReleaseHandle;
 polyfound:=false;
 if (assigned(FOnHotClick) and (ssLeft in Shift)) or ((ssRight in Shift)) then
 begin
   for i:=0 to self.Polygons.Count-1 do
   begin
      npoints:=self.Polygons.Items[i].Poly.Count;
      if npoints>100 then npoints:=100;
      for j:=0 to npoints-1 do
      begin
         PolyPoints[j].X:=self.Polygons.Items[i].Poly.Items[j].Point.X;
         PolyPoints[j].Y:=self.Polygons.Items[i].Poly.Items[j].Point.Y;
      end;
      HotSpotRgn := CreatePolygonRgn(PolyPoints,npoints, WINDING);
      if PtInRegion(HotSpotRgn, X,Y) then
      begin
         if (ssLeft in Shift) then
            FOnHotClick(self,I);
         polyfound:=true;
         DeleteObject(HotSpotRgn);
         Polygon:=-1;
         break;
      end;
      DeleteObject(HotSpotRgn);
   end;
   if not polyfound then
      inherited
   else if (ssRight in Shift) and (self.Polygons[I].Poly.PopupMenu<>nil) then
        begin
            Repaint;
            punto:=clienttoscreen(point(x,y));
            self.Polygons[I].Poly.PopupMenu.Popup(punto.X,punto.y);
        end
        else
            inherited;
 end
 else
 inherited;
end;

procedure THotSpotMapN.MouseMove(Shift: TShiftState; X, Y: Integer);
var
   HotSpotRgn: HRGN;
   PolyPoints: array[0..100] of TPoint;
   i,j:integer;
   nPoints:Integer;
   R:TRect;
   polyfound:boolean;
   P:TPoint;
begin
 p.X:=X;
 p.Y:=Y;
 polyfound:=false;
 self.Cursor:=crDefault;
 for i:=0 to self.Polygons.Count-1 do
 begin
    npoints:=self.Polygons.Items[i].Poly.Count;
    if npoints>100 then npoints:=100;
      for j:=0 to npoints-1 do
      begin
         PolyPoints[j].X:=self.Polygons.Items[i].Poly.Items[j].Point.X;
         PolyPoints[j].Y:=self.Polygons.Items[i].Poly.Items[j].Point.Y;
      end;
    HotSpotRgn := CreatePolygonRgn(PolyPoints,npoints, WINDING);
    if PtInRegion(HotSpotRgn, X, Y) then
    begin
       self.Cursor:=crHandPoint;
       DeleteObject(HotSpotRgn);
       if ShowHint and (self.Polygons[i].Poly.Hint<>'') then
       begin
          p:=self.clienttoscreen(p);
          r:=FHintw.CalcHintRect(200,Polygons[i].Poly.Hint,nil);
          offsetRect(R,p.X,p.y+32);
          self.FHintW.Color:=Polygons[i].Poly.HintColor;
          self.FHintW.ActivateHint(R ,Polygons[i].Poly.Hint);
       end;
       polyfound:=true;
       Polygon:=i;
       break;
    end;
    DeleteObject(HotSpotRgn);
 end;
 if not polyfound and fhintw.HandleAllocated then
 begin
     FHintW.ReleaseHandle;
     Polygon:=-1;
 end;
 inherited
end;

constructor TPPoint.Create;
begin
   FX:=0;
   FY:=0;
end;

constructor TPPointitem.Create(Collection:TCollection);
begin
   inherited;
   Fpoint:=TPPoint.Create;
end;
function TPPointItem.GetDisplayName: string;
begin
   result:='Point_'+inttostr(index);
end;
constructor TPoly.Create(Owner:TPersistent);
begin
   inherited Create(TPPointItem);
   Fowner:=owner;
   FHintColor:=clInfoBk;
end;
function TPoly.GetItem(Index:Integer):TPPointItem;
begin
   result:=TPPointItem(inherited GetItem(Index));
end;
Procedure TPoly.SetItem(Index:Integer;Value:TPPointItem);
begin
   inherited SetItem(Index,Value);
end;
Function TPoly.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TPoly.Add:TPPointItem;
begin
   result:=TPPointItem(Inherited Add);
end;

constructor TPolyitem.Create(Collection:TCollection);
begin
   inherited;
   Fpoly:=TPoly.Create(self);
end;
function TPolyItem.GetDisplayName: string;
begin
   result:='Poly_'+inttostr(index);
end;
constructor TPolys.Create(Owner:TPersistent);
begin
   inherited Create(TPolyItem);
   Fowner:=owner;
   FLoadFrom:='(File)';
end;
function TPolys.GetItem(Index:Integer):TPolyItem;
begin
   result:=TPolyItem(inherited GetItem(Index));
end;
Procedure TPolys.SetItem(Index:Integer;Value:TPolyItem);
begin
   inherited SetItem(Index,Value);
end;
Function TPolys.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TPolys.Add:TPolyItem;
begin
   result:=TPolyItem(Inherited Add);
end;
end.
