unit HotSpotMap;

interface

uses
  Windows, Messages, SysUtils, Classes,Graphics,ExtCtrls,controls,dialogs,menus;

type
  TPPoint=class(TPersistent)
  private
     FX:Integer;
     FY:Integer;
  public
     Constructor Create;
  published
     property X:Integer read FX write FX;
     property Y:Integer read FY write FY;
  end;

  TPPointItem=class(TCollectionItem)
     private
        FPoint:TPPoint;
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Point:TPPoint read FPoint Write FPoint;
  end;

  TPoly=Class(TCollection)
     private
        FOwner:TPersistent;
        FHint:String;
        FHintColor:TColor;
        FPopupMenu:TPopupMenu;
        function GetItem(Index:Integer):TPPointItem;
        Procedure SetItem(Index:Integer;Value:TPPointItem);
     protected
        function GetOwner:TPersistent;override;
     public
        Constructor Create(Owner:TPersistent);
        Function Add:TPPointItem;
        property Items[Index:Integer]:TPPointItem read Getitem write SetItem; default;
     published
        property Hint:String read FHint write FHint;
        property HintColor:TColor read FHintColor write FHintColor default clInfoBk;
        property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu;
  end;

  TPolyItem=class(TCollectionItem)
     private
        FPoly:TPoly;
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
     published
        property Poly:TPoly read FPoly Write FPoly;
  end;

  TPolys=Class(TCollection)
     private
        FOwner:TPersistent;
        FLoadFrom:TFilename;
        function GetItem(Index:Integer):TPolyItem;
        Procedure SetItem(Index:Integer;Value:TPolyItem);
     protected
        function GetOwner:TPersistent;override;
     public
         Constructor Create(Owner:TPersistent);
         Function Add:TPolyItem;
         property Items[Index:Integer]:TPolyItem read Getitem write SetItem; default;
     published
  end;

  TOnHotClick=procedure(Sender : TObject;Polygon:integer) of object;
  TOnPolygonActive=TOnHotClick;
  THotSpotMap = class(TImage)
  private
    FPolygons:TPolys;
    FOnHotClick:TOnHotClick;
    FOnPolygonActive:TOnPolygonActive;
    FHintW:THintWindow;
    FPolygon:integer;
    FEventOutPolygon:Boolean;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure SetPicture(Value:TPicture);
    function GetPicture:TPicture;
    procedure SetPolygon(Value:Integer);
    { Private declarations }
  protected
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);override;
     procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
     procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    { Protected declarations }
  public
    constructor Create(AOwner:TComponent);override;
    Destructor Destroy;override;
    property Polygon:Integer read FPolygon write SetPolygon default -1;
    { Public declarations }
  published
    Property Polygons:TPolys read FPolygons write FPolygons;
    Property OnHotClick:TOnHotClick read FOnHotClick write FOnHotClick;
    Property Picture:TPicture read GetPicture write setpicture;
    property OnPolygonActive:TOnPolygonActive read FOnPolygonActive write FOnPolygonActive;
    property EventOutPolygon:Boolean read FEventOutPolygon write FEventOutPolygon default false;
    { Published declarations }
  end;

implementation
constructor THotSpotMap.Create(AOwner:TComponent);
begin
   inherited;
   FPolygons:=TPolys.Create(self);
   FHintW:=THintWindow.Create(self);
   FPolygon:=-1;
   FEventOutPolygon:=false;
end;
Destructor THotSpotMap.Destroy;
begin
   FPolygons.Free;
   FHintW.Free;
   inherited;
end;
procedure THotSpotMap.Notification(AComponent: TComponent;
      Operation: TOperation);
var
   i:integer;
begin
   inherited;
   if (AComponent.classtype=TPopupMenu) and (Operation=opRemove) then
   begin
      for i:=0 to self.polygons.count-1 do
         if self.Polygons[i].Poly.PopupMenu=AComponent then
            self.Polygons[i].Poly.PopupMenu:=nil;
   end;
end;

procedure THotSpotMap.SetPolygon(Value:Integer);
begin
   if FPolygon<>value then
   begin
      FPolygon:=Value;
      if assigned(FOnPolygonActive) and (((value=-1) and EventOutPolygon) or (value<>-1)) then
         FOnPolygonActive(self,Value)
   end;
end;
procedure THotSpotMap.SetPicture(Value:TPicture);
var
   tmp:TBitmap;
begin
   tmp:=tbitmap.Create;
   Tmp.Width:=value.Width;
   Tmp.Height:=value.Height;
   Tmp.Canvas.Draw(0,0,value.Graphic);
   Picture.Assign(Tmp);
   tmp.Free;
end;
function THotSpotMap.getpicture:TPicture;
begin
   result:=inherited Picture;
end;
procedure THotSpotMap.CMMouseLeave(var Message: TMessage);
begin
  if fhintw.HandleAllocated then
     FHintw.ReleaseHandle;
  if Parent <> nil then
    Parent.Perform(CM_MOUSELEAVE, 0, Longint(Self));
  Polygon:=-1;
end;
procedure THotSpotMap.MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer);
var
   HotSpotRgn: HRGN;
   PolyPoints: array[0..100] of TPoint;
   i,j:integer;
   polyfound:boolean;
   npoints:integer;
   punto:tpoint;
begin
 if fhintw.HandleAllocated then
    FHintW.ReleaseHandle;
 polyfound:=false;
 if (assigned(FOnHotClick) and (ssLeft in Shift)) or ((ssRight in Shift)) then
 begin
   for i:=0 to self.Polygons.Count-1 do
   begin
      npoints:=self.Polygons.Items[i].Poly.Count;
      if npoints>100 then npoints:=100;
      for j:=0 to npoints-1 do
      begin
         PolyPoints[j].X:=self.Polygons.Items[i].Poly.Items[j].Point.X;
         PolyPoints[j].Y:=self.Polygons.Items[i].Poly.Items[j].Point.Y;
      end;
      HotSpotRgn := CreatePolygonRgn(PolyPoints,npoints, WINDING);
      if PtInRegion(HotSpotRgn, X,Y) then
      begin
         if (ssLeft in Shift) then
            FOnHotClick(self,I);
         polyfound:=true;
         DeleteObject(HotSpotRgn);
         Polygon:=-1;
         break;
      end;
      DeleteObject(HotSpotRgn);
   end;
   if not polyfound then
      inherited
   else if (ssRight in Shift) and (self.Polygons[I].Poly.PopupMenu<>nil) then
        begin
            punto:=clienttoscreen(point(x,y));
            self.Polygons[I].Poly.PopupMenu.Popup(punto.X,punto.y);
        end
        else
            inherited;
 end
 else
 inherited;
end;

procedure THotSpotMap.MouseMove(Shift: TShiftState; X, Y: Integer);
var
   HotSpotRgn: HRGN;
   PolyPoints: array[0..100] of TPoint;
   i,j:integer;
   nPoints:Integer;
   R:TRect;
   polyfound:boolean;
   P:TPoint;
begin
 p.X:=X;
 p.Y:=Y;
 polyfound:=false;
 self.Cursor:=crDefault;
 for i:=0 to self.Polygons.Count-1 do
 begin
    npoints:=self.Polygons.Items[i].Poly.Count;
    if npoints>100 then npoints:=100;
      for j:=0 to npoints-1 do
      begin
         PolyPoints[j].X:=self.Polygons.Items[i].Poly.Items[j].Point.X;
         PolyPoints[j].Y:=self.Polygons.Items[i].Poly.Items[j].Point.Y;
      end;
    HotSpotRgn := CreatePolygonRgn(PolyPoints,npoints, WINDING);
    if PtInRegion(HotSpotRgn, X, Y) then
    begin
       self.Cursor:=crHandPoint;
       DeleteObject(HotSpotRgn);
       if ShowHint and (self.Polygons[i].Poly.Hint<>'') then
       begin
          p:=self.clienttoscreen(p);
          r:=FHintw.CalcHintRect(200,Polygons[i].Poly.Hint,nil);
          offsetRect(R,p.X,p.y+32);
          self.FHintW.Color:=Polygons[i].Poly.HintColor;
          self.FHintW.ActivateHint(R ,Polygons[i].Poly.Hint);
       end;
       polyfound:=true;
       Polygon:=i;
       break;
    end;
    DeleteObject(HotSpotRgn);
 end;
 if not polyfound and fhintw.HandleAllocated then
 begin
     FHintW.ReleaseHandle;
     Polygon:=-1;
 end;
 inherited
end;

constructor TPPoint.Create;
begin
   FX:=0;
   FY:=0;
end;

constructor TPPointitem.Create(Collection:TCollection);
begin
   inherited;
   Fpoint:=TPPoint.Create;
end;
function TPPointItem.GetDisplayName: string;
begin
   result:='Point_'+inttostr(index);
end;
constructor TPoly.Create(Owner:TPersistent);
begin
   inherited Create(TPPointItem);
   Fowner:=owner;
   FHintColor:=clInfoBk;
end;
function TPoly.GetItem(Index:Integer):TPPointItem;
begin
   result:=TPPointItem(inherited GetItem(Index));
end;
Procedure TPoly.SetItem(Index:Integer;Value:TPPointItem);
begin
   inherited SetItem(Index,Value);
end;
Function TPoly.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TPoly.Add:TPPointItem;
begin
   result:=TPPointItem(Inherited Add);
end;

constructor TPolyitem.Create(Collection:TCollection);
begin
   inherited;
   Fpoly:=TPoly.Create(self);
end;
function TPolyItem.GetDisplayName: string;
begin
   result:='Poly_'+inttostr(index);
end;
constructor TPolys.Create(Owner:TPersistent);
begin
   inherited Create(TPolyItem);
   Fowner:=owner;
   FLoadFrom:='(File)';
end;
function TPolys.GetItem(Index:Integer):TPolyItem;
begin
   result:=TPolyItem(inherited GetItem(Index));
end;
Procedure TPolys.SetItem(Index:Integer;Value:TPolyItem);
begin
   inherited SetItem(Index,Value);
end;
Function TPolys.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TPolys.Add:TPolyItem;
begin
   result:=TPolyItem(Inherited Add);
end;
end.
