unit LabelEdit;

interface

uses
  Windows, Messages,graphics, SysUtils, Classes, Controls, StdCtrls,dialogs,math,Forms;

type
  //Procedimiento para OnPreEdit y OnPostedit
  TEditEvent=Procedure(Sender:TObject;var TextEdit:String) of object;
  // Propiedades del Edit
  TEditStyle=Class(TPersistent)
  private
    FWantReturn:Boolean;
    FCtl3D:boolean;
    FMinWidth,
    FMinHeight:Integer;
    FBorderStyle:TBorderStyle;
    FColor:TColor;
    FFont:TFont;
    FMaxLength:Integer;
    procedure SetMinHeight(value:Integer);
  protected
  public
     constructor Create;
     Destructor destroy;override;
  published
    property Color:TColor read FColor write FColor default clWindow;
    property WantReturn:Boolean read FWantReturn write FWantReturn default false;
    property Ctl3D:boolean read FCtl3D write FCtl3D default True;
    property MinWidth:Integer read FMinWidth write FMinWidth default 0;
    property MinHeight:Integer read FMinHeight write SetMinHeight default 21;
    property BorderStyle:TBorderStyle read FBorderStyle write FBorderStyle;
    property Font:TFont read FFont write FFont;
    property MaxLength:Integer read FMaxLength write FMaxLength;
  end;
  TLabelEdit = class(TCustomLabel)
  private
    FEdit:TMemo;
    FOrigWindowProc : TWndMethod;
    FEditStyle:TEditStyle;
    FEditable:Boolean;
    FOnPreEdit:TEditEvent;
    FOnPostEdit:TEditEvent;
    FOnChange:TNotifyEvent;
    FOnKeyDown: TKeyEvent;
    FOnKeyPress: TKeyPressEvent;
    FOnKeyUp: TKeyEvent;
    Procedure ControlSubClass(var Message:TMessage);
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    Procedure CreateEdit;
  protected
  public
     constructor create(aowner:TComponent);override;
     destructor Destroy;override;
     procedure Edit;
     procedure Close;
  published
    property EditStyle:TEditStyle read FEditStyle write FEditStyle;
    property Editable :Boolean read FEditable Write FEditable default True;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
    property OnChange:TNotifyEvent read FOnChange write FOnChange;
    property OnPreEdit : TEditEvent read FOnPreEdit write FOnPreEdit;
    property OnPostEdit : TEditEvent read FOnPostEdit write FOnPostEdit;
    property OnKeyDown: TKeyEvent read FOnKeyDown write FOnKeyDown;
    property OnKeyPress: TKeyPressEvent read FOnKeyPress write FOnKeyPress;
    property OnKeyUp: TKeyEvent read FOnKeyUp write FOnKeyUp;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('i-vcl Labels', [TLabelEdit]);
end;
constructor TEditStyle.Create;
begin
   inherited;
   FCtl3D:=True;
   FWantReturn:=false;
   FMinHeight:=21;
   FMinWidth:=0;
   FBorderStyle:=bsSingle;
   FColor:=clWindow;
   FFont:=tfont.Create;
end;
Destructor TEditStyle.destroy;
begin
   ffont.free;
   inherited;
end;
procedure TEditStyle.SetMinHeight(Value:Integer);
begin
   if (value<>FMinHeight) and (value>=0) then
      FMinHeight:=value;
end;
constructor TLabelEdit.Create(AOwner:TComponent);
begin
   inherited;
   FEditStyle:=TEditStyle.Create;
   FEditable:=True;
end;
destructor TLabelEdit.destroy;
begin
   if assigned(FOrigWindowProc) then
   begin
      if assigned(Fedit) then
      begin
         fEdit.WindowProc:=FOrigWindowProc;
         FOrigWindowProc:=nil;
      end;
   end;
   fEditStyle.Free;
   inherited;
end;
procedure TLabelEdit.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  inherited;
  CreateEdit;
end;
Procedure TLabelEdit.ControlSubClass(var Message:TMessage);
begin
   if message.Msg=WM_CHAR then
   begin
      if (Chr(Message.wParam) = #13) then
      begin
         if FEditStyle.WantReturn then
            FOrigWindowProc(message);
      end
      else
         FOrigWindowProc(message);
   end
   else
   begin
   FOrigWindowProc(message);
   if message.Msg=CM_FOCUSCHANGED then
   begin
      self.close;
      message.Result:=-1;
   end;
   end;
end;
procedure TLabelEdit.Close;
var
   t:string;
begin
      t:=fedit.text;
      if assigned(FOnPostEdit) then FOnPostEdit(self,t);
      caption:=t;
      fedit.WindowProc:=FOrigWindowProc;
      FOrigWindowProc:=nil;
{      if assigned(FEdit.onchange) then
         FEdit.OnChange:=nil;
      if Assigned(FOnKeyDown) then
         FEdit.OnKeyDown:=FOnKeyDown;
      if Assigned(FOnKeyUp) then
         FEdit.OnKeyUp:=FOnKeyUp;
      if Assigned(FOnKeyPress) then
         FEdit.OnKeyPress:=FOnKeyPress;}
      fedit.Free;
      fedit:=nil;
end;
procedure TLabelEdit.CreateEdit;
var
   t:string;
begin
  if not assigned(FEdit) and FEditable then
  begin
     t:=caption;
     if assigned(FOnPreEdit) then FOnPreEdit(self,t);
     fedit:=tMemo.Create(self);
     with FEdit do
     begin
        Text:=t;     
        if assigned(FOnChange) then
           FEdit.OnChange:=FOnChange;
        if Assigned(FOnKeyDown) then
           FEdit.OnKeyDown:=FOnKeyDown;
        if Assigned(FOnKeyUp) then
           FEdit.OnKeyUp:=FOnKeyUp;
        if Assigned(FOnKeyPress) then
           FEdit.OnKeyPress:=FOnKeyPress;
        Top:=self.Top;
        Left:=self.Left;
        Height:=max(self.Height,FEditStyle.MinHeight);
        Width:=max(self.Width,FEditStyle.MinWidth);
        Font:=FEditStyle.Font;
        ParentCtl3D:=false;
        BorderStyle:=FEditStyle.BorderStyle;
        Ctl3D:=FEditStyle.ctl3d;
        Color:=FEditStyle.Color;
        fedit.MaxLength:=FEditStyle.MaxLength;
        Visible:=true;
        Parent:=self.Parent;
        SetFocus;
     end;
     FOrigWindowProc:= fedit.WindowProc;
     fedit.WindowProc:=ControlSubClass;
  end;
end;
procedure TLabelEdit.Edit;
begin
   CreateEdit;
end;
end.
