unit NavBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls,Buttons,
  StdCtrls,igriegastrs,funciones,imglist,graphics,Dialogs;

type
  TNavBarBtn = (nbFirst, nbPrior, nbNext, nbLast);
  TEditNavBar=Class(TEdit)
  private
      FOnNumberChanged:TNotifyEvent;
      procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Press(Sender:TObject;var Key: Char);
    procedure SetEditText;
  public
    property OnNumberChanged:TNotifyEvent read FOnNumberChanged write FOnNumberChanged;
  end;
  TNavBar = class(TCustomPanel)
  private
    FButtons:array[0..3] of TSpeedButton;
    FEdit:TEditNavBar;
    FFlat:Boolean;
    FPosition,
    FMax,
    FMin:Integer;
    FMask:TNavBarMask;
    FReadOnly:Boolean;
    FImageChangeLink: TChangeLink;
    FImageList:TImageList;
    FOnChange:TNotifyEvent;
    FButtonWidth:Integer;
    FNumGlyphs:integer;
    procedure SetNumGlyphs(Value:Integer);
    procedure SetButtonWidth(Value:Integer);
    procedure SetFlat(Value:Boolean);
    procedure SetMax(Value:Integer);
    procedure SetMin(Value:Integer);
    procedure SetPosition(Value:Integer);
    procedure SetParam(APosition,AMin,AMax:integer);
    procedure SetReadOnly(Value:Boolean);
    Procedure SetMask(Value:TNavBarMask);
    Function Distance:Integer;
    procedure SetText;
    procedure btnClick(Sender:TObject);
    procedure TextChanged(Sender:TObject);
    procedure SetImageList(value:TImageList);
    procedure ImageListChanged(Sender:TObject);
    procedure SetDefaultGlyph;
    procedure SetListGlyph;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    { Private declarations }
  protected
     procedure Notification (AComponent: TComponent; Operation: TOperation); override;
    { Protected declarations }
  public
     constructor Create(Aowner:TComponent);OVERRIDE;
     destructor Destroy;override;
    { Public declarations }
  published
    property NumGlyphs:Integer read FNumGlyphs write SetNumGlyphs default 2;
    property ButtonWidth:integer read FButtonWidth write SetButtonWidth default 23;
    property Flat:Boolean read FFlat write SetFlat;
    property Position:integer read FPosition write SetPosition default 1;
    property Max:integer read FMax write SetMax default 100;
    property Min:integer read FMin write SetMin default 0;
    property Mask:TNavBarMask read FMask write SetMask;
    Property ImageList:TImageList read FImageList write SetImageList;
    property ReadOnly:Boolean read FReadOnly write SetReadOnly default false;
    property Align;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange:TNotifyEvent read FOnChange write FOnchange;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    { Published declarations }
  end;
procedure Register;

implementation

{$R NAVBUTTONS}
var
   BtnTypeName: array[0..3] of PChar = ('FIRST','NEXT', 'PRIOR', 'LAST');
function IsNumber(s: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 1 to Length(s) do
    case s[i] of
      '0'..'9':;
      else
        Exit;
    end;
  Result := True;
end;
procedure TEditNavBar.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_CENTER or es_number;
end;
procedure TEditNavBar.Press(Sender:TObject;var Key: Char);
begin
   if not (isnumber(self.Text)) then
      self.SelectAll;
   if key=#13 then
   begin
      key:=#0;
      SetEditText;
   end;
end;
procedure TEditNavBar.SetEditText;
begin
   if assigned(FOnNumberChanged) then
       OnNumberChanged(Self);
end;
procedure TEditNavBar.WMKillFocus(var Message: TWMSetFocus);
begin
   SetEditText;
   inherited;
end;
procedure TNavBar.SetNumGlyphs(Value:Integer);
begin
   if (value>0) and (Value<>FNumGlyphs) then
   begin
      FNumGlyphs:=Value;
      if assigned(imagelist) then
         SetListGlyph;
   end;
end;
procedure TNavBar.SetListGlyph;
var
  i:integer;
  t:tbitmap;
begin
   t:=tbitmap.Create;
   if ImageList.Count>=4 then
   begin
      for i:=0 to 3 do
      begin
         imagelist.GetBitmap(i,t);
         FButtons[i].Glyph.Assign(t);
         Fbuttons[i].NumGlyphs:=NumGlyphs;
      end;
   end
   else
      SetDefaultGlyph;
   t.free;
end;
procedure TNavBar.SetDefaultGlyph;
var
   i:integer;
   ResName: string;
begin
   for i:=0 to 3 do
   begin
      FmtStr(ResName, 'b_%s', [BtnTypeName[I]]);
      FButtons[i].Glyph.LoadFromResourceName(HInstance, ResName);
      FButtons[i].NumGlyphs := 2;
   end;
end;
constructor TNavBar.Create(Aowner:TComponent);
const
   v_align:array[0..3] of talign=(alLeft,alRight,alLeft,alRight);
var
   i:integer;
   ResName: string;
begin
   inherited;
   self.Caption:='';
   self.Height:=22;
   FMax:=100;
   FMin:=1;
   FPosition:=1;
   FMask:= sNavBarDefaultMask;
   FReadOnly:=false;
   FButtonWidth:=23;
   FNumGlyphs:=2;
   for i:=0 to 3 do
   begin
      FButtons[i]:=tspeedbutton.Create(self);
      FButtons[i].Width:=FButtonWidth;
      FButtons[i].Parent:=self;
      FButtons[i].Visible:=true;
      FButtons[i].Align:=v_align[i];
      FmtStr(ResName, 'b_%s', [BtnTypeName[I]]);
      FButtons[i].Name:=ResName;
      FButtons[i].OnClick:=btnClick;
   end;
   SetDefaultGlyph;
   FEdit:=TEditNavBar.Create(self);
   FEdit.Parent:=self;
   FEdit.Visible:=true;
   FEdit.Align:=alClient;
   FEdit.OnKeyPress:=Fedit.Press;
   FEdit.OnNumberChanged:=textchanged;
   SetText;
end;
destructor TNavBar.Destroy;
var
   i:integer;
begin
   for i:=0 to 3 do
      FButtons[i].Free;
   FEdit.Free;
   if assigned(FImageChangeLink) then
      FImageChangeLink.free;
   inherited;
end;
procedure TNavBar.SetButtonWidth(Value:Integer);
var i:integer;
begin
   if (value>0) and (value<>FButtonWidth) then
   begin
      FButtonWidth:=Value;
      for i:=0 to 3 do
         self.FButtons[i].Width:=Value;
   end;
end;
procedure TNavBar.CMEnabledChanged(var Message: TMessage);
var
  i:integer;
begin
   inherited;
   for i:=0 to 3 do
      fbuttons[i].Enabled:=Enabled;
   fedit.Enabled:=Enabled;
end;
procedure TNavBar.btnClick(Sender:TObject);
begin
   case tspeedbutton(sender).Name[3] of
      'F':position:=min;
      'P':position:=position-1;
      'N':position:=position+1;
      'L':position:=max;
   end;
end;
procedure TNavBar.TextChanged(Sender:TObject);
begin
   if isnumber(fedit.Text) then
      position:=Strtoint(fedit.Text);
end;
procedure TNavBar.SetFlat(Value:Boolean);
var i:integer;
begin
   if FFlat<>Value then
   begin
      fflat:=value;
      for i:=0 to 3 do
         fbuttons[i].Flat:=fflat;
      invalidate;
   end;
end;
procedure TNavBar.SetParam(APosition,AMin,AMax:Integer);
begin
  if APosition < AMin then APosition := AMin;
  if APosition > AMax then APosition := AMax;
  if (FMin <> AMin) then FMin := AMin;
  if (FMax <> AMax) then FMax := AMax;
  if FPosition <> APosition then FPosition := APosition;
  settext;
end;
procedure TNavBar.SetPosition(Value:Integer);
var
  p:integer;
begin
  p:=position;
  setparam(Value,FMin,FMax);
  if p<>position then
     if assigned(FOnchange) then FOnChange(Self);
end;
procedure TNavBar.SetMax(Value:Integer);
begin
   if Value >= FMin then
   begin
      setparam(FPosition,FMin,Value);
   end;
end;
procedure TNavBar.SetMin(Value:Integer);
begin
   if Value <= FMax then
   begin
      setparam(FPosition,Value,FMax);
   end;
end;
Function TNavBar.Distance:Integer;
begin
   result:=FMax-FMin +1;
end;
procedure TNavBar.SetText;
var
   t:string;
begin
   t:=Mask;
   t:=stringreplace(t,'%d%',inttostr(distance),[rfReplaceAll,rfIgnoreCase]);
   t:=stringreplace(t,'%p%',inttostr(Position),[rfReplaceAll,rfIgnoreCase]);
   t:=stringreplace(t,'%f%',inttostr(Min),[rfReplaceAll,rfIgnoreCase]);
   t:=stringreplace(t,'%e%',inttostr(Max),[rfReplaceAll,rfIgnoreCase]);
   FEdit.Text:=t;
end;
Procedure TNavBar.SetMask(Value:TNavBarMask);
begin
   FMask:=Value;
   SetText;
end;
procedure TNavBar.SetReadOnly(Value:Boolean);
begin
   if FReadOnly<>Value then
   begin
      FReadOnly:=Value;
      FEdit.ReadOnly:=Value;
   end;
end;
//------------
procedure TNavBar.SetImageList(Value:TImageList);
begin
    if FImageList <> Value then
    begin
      if FImageList <> nil then
        FImageList.UnRegisterChanges (FImageChangeLink);
      FImageList := Value;
      if FImageList <> nil then
      begin
        if FImageChangeLink = nil then
        begin
          FImageChangeLink := TChangeLink.Create;
          FImageChangeLink.OnChange := ImageListChanged;
        end;
        FImageList.RegisterChanges (FImageChangeLink);
        FImageList.FreeNotification (Self);
        SetListGlyph;
      end
      else
      begin
        FImageChangeLink.Free;
        FImageChangeLink := nil;
        fimagelist:=nil;
        SetDefaultGlyph
      end
    end;
end;
procedure TNavBar.Notification (AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if Assigned(FImagelist) and (AComponent = ImageList) then
    begin
       ImageList := nil;
       SetDefaultGlyph
    end;
  end;
end;
procedure TNavBar.ImageListChanged(Sender:TObject);
begin
   setimagelist(timagelist(sender));
   SetListGlyph;
end;

//--------------

procedure Register;
begin
  RegisterComponents('i-griegavcl', [TNavBar]);
end;

end.

