unit OverComp;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls,graphics;

type
  TOverComp = class(tcustomcontrol)
  private
     FColors:array[0..5] of TColor;
     FOver:Boolean;
     FOnMouseEnter,
     FOnMouseLeave:TNotifyEvent;
     FBorder:Boolean;
     FBorderWidth:integer;
     FPicture:TPicture;
     FPosXPicture:Word;
     FPosXText:Word;
     FText:TCaption;
     FTextFont:TFont;
     procedure SetColors(Index:Integer;Value:TColor);
     function GetColors(Index:integer):TColor;
     procedure SetBorder(Value:Boolean);
     procedure SetPicture(Value:TPicture);
     procedure SetBorderWidth(Value:integer);
     procedure SetPosXPicture(Value:Word);
     procedure SetPosXText(Value:Word);
     procedure SetText(Value:TCaption);
     procedure SetTextFont(Value:TFont);
     procedure FontChanged(Sender: TObject);
    { Private declarations }
  protected
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure KeyPress(var Key: Char);override;
    procedure Paint; override;
    procedure Click;override;
    { Protected declarations }
  public
     constructor Create(AOwner:TComponent);override;
     destructor Destroy;override;
     property Colors[Index:Integer]:TColor read GetColors Write SetColors;
    { Public declarations }
  published
     property Color:TColor Index 0 read GetColors Write SetColors default clBtnFace;
     property BorderColor:TColor Index 1 read GetColors Write SetColors default clBlack;
     property FocusedColor:TColor Index 2 read GetColors Write SetColors default clBtnHighlight;
     property FocusedBorderColor:TColor Index 3 read GetColors Write SetColors default clBlack;
     property OverColor:TColor Index 4 read GetColors Write SetColors default clBtnShadow;
     property OverBorderColor:TColor Index 5 read GetColors Write SetColors default clBlack;
     property Border:Boolean read FBorder Write SetBorder default True;
     property BorderWidth:integer read FBorderWidth Write SetBorderWidth default 1;
     property Picture:TPicture read FPicture Write SetPicture;
     property PosXPicture:Word read FPosXPicture Write SetPosXPicture default 10;
     property PosXText:Word read FPosXText Write SetPosXText default 50;
     property Caption;
     property Text:TCaption read FText Write SetText;
     property TextFont:TFont read FTextFont Write SetTextFont;
     property OnMouseEnter:TNotifyEvent read FOnMouseEnter Write FOnMouseEnter;
     property OnMouseLeave:TNotifyEvent read FOnMouseLeave Write FOnMouseLeave;
     property TabStop;
     property TabOrder;
     property Action;
     property Align;
     property Anchors;
     property Visible;
     property Enabled;
     property Constraints;
     property DragCursor;
     property DragKind;
     property DragMode;
     property Font;
     property ParentBiDiMode;
     property ParentFont;
     property ParentShowHint;
     property PopupMenu;
     property ShowHint;
     property OnClick;
     property OnContextPopup;
     property OnDblClick;
     property OnDragDrop;
     property OnDragOver;
     property OnEndDock;
     property OnEndDrag;
     property OnMouseDown;
     property OnMouseMove;
     property OnMouseUp;
     property OnStartDock;
     property OnStartDrag;
     property OnEnter;
     property OnExit;

    { Published declarations }
  end;

procedure Register;

implementation

constructor TOverComp.Create(AOwner:TComponent);
begin
   inherited;
   FPicture:=TPicture.Create;
   Self.Width:=160;
   FTextFont:=TFont.Create;
   FTextFont.OnChange:=FontChanged;
   FTextFont.Color:=$00505050;
   FTextFont.Style:=[fsBold];
   FColors[0]:= clBtnFace;
   FColors[1]:=clBlack;
   FColors[2]:=clBtnHighlight;
   FColors[3]:=clBlack;
   FColors[4]:= clBtnShadow;
   FColors[5]:=clBlack;
   TabStop:=True;
   Visible:=True;
   Enabled:=True;
   ControlStyle:=[csSetCaption,csClickEvents,csCaptureMouse,csDoubleClicks,csReflector,csActionClient,csOpaque];
   FBorder:=True;
   FBorderWidth:=1;
   FPosXPicture:=10;
   FPosXText:=50;
   FText:='';
   Font.Style:=[fsBold];
end;
destructor TOverComp.Destroy;
begin
   if Assigned(Picture) then
      Picture.Free;
   FTextFont.Free;
   inherited;
end;
procedure TOverComp.SetPicture(Value:TPicture);
begin
   FPicture.Assign(Value);
   repaint;
end;
procedure TOverComp.Paint;
var
   R:TRect;
   Flags:Integer;
   despv,desph:Integer;
   X, Y, W, H: Integer;
begin
   Desph:=BorderWidth-1;
   despv:=3;
   if border then
      despv:=despv+BorderWidth;
   Flags:=DT_WORDBREAK or DT_LEFT or DT_NOPREFIX;
   with Canvas do
   begin
      Font:=Self.Font;
      Pen.Width:=BorderWidth;
      Pen.Color:=BorderColor;
      Brush.Style:=bsSolid;
      Brush.Color:=Color;
      X := Pen.Width div 2;
      Y := X;
      W := Width - Pen.Width + 1;
      H := Height - Pen.Width + 1;
      if Focused then
      begin
         Pen.Color:=FocusedBorderColor;
         Brush.Color:=FocusedColor;
      end;
      if FOver then
      begin
         Pen.Color:=OverBorderColor;
         Brush.Color:=OverColor;
      end;
      FillRect(ClientRect);
      if Assigned(Picture.Graphic) then
      begin
         Picture.Graphic.Transparent:=True;
         if Picture.Graphic.ClassName='TBitmap' then
            Picture.Bitmap.TransparentMode:=tmAuto;
         Draw(PosXPicture+desph,((Height-Picture.Graphic.Height) div 2),Picture.Graphic);
      end;
      setbkmode(Handle,TRANSPARENT);
      TextOut(PosXText+desph,despv,Caption);
      R:=Rect(posxText+desph,TextHeight(Caption)+despv,width-despv,height-despv);
      font:=TextFont;
      DrawText(Handle, PChar(Text), -1, R, Flags);
      Brush.Style:=bsClear;
      if Border then Rectangle(X, Y, X + W, Y + H);
   end;
end;

procedure TOverComp.WMSetFocus(var Message: TWMSetFocus);
begin
   inherited;
   Invalidate;
end;

procedure TOverComp.WMKillFocus(var Message: TWMSetFocus);
begin
   inherited;
   Invalidate;
end;

procedure TOverComp.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnmouseEnter) then FOnMouseEnter(Self);
  FOver:=True;
  Invalidate;
end;

procedure TOverComp.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnmouseLeave) then FOnMouseLeave(Self);
  FOver:=False;
  Invalidate;
end;
procedure TOverComp.KeyPress(var Key: Char);
begin
   if (Key=#13) or (Key=#32) then
      Click;
   Key:=#0;
end;
procedure  TOverComp.Click;
begin
   inherited;
   SetFocus;
end;
procedure TOverComp.SetColors(Index:Integer;Value:TColor);
begin
   if FColors[Index]<>Value then
   begin
      FColors[Index]:=Value;
      Invalidate;
   end;
end;
Function TOverComp.GetColors(Index:Integer):TColor;
begin
   Result:=FColors[Index];
end;
procedure TOverComp.SetBorder(Value:Boolean);
begin
   if FBorder<>Value then
   begin
      FBorder:=Value;
      Invalidate;
   end;
end;

procedure TOverComp.SetBorderWidth(Value:integer);
begin
   if FBorderWidth<>Value then
   begin
      if Value>0 then
         FBorderWidth:=Value;
      Invalidate;
   end;
end;
procedure TOverComp.SetPosXPicture(Value:Word);
begin
   if FPosXPicture<>Value then
   begin
      FPosXPicture:=Value;
      invalidate;
   end;
end;
procedure TOverComp.SetPosXText(Value:Word);
begin
   if FPosXText<>Value then
   begin
      FPosXText:=Value;
      invalidate;
   end;
end;
procedure TOverComp.SetText(Value: TCaption);
begin
   if FText<>Value then
   begin
      FText:=Value;
      invalidate;
   end;
end;
procedure TOverComp.CMTextChanged(var Message: TMessage);
begin
   inherited;
   invalidate;
end;
procedure TOverComp.SetTextFont(Value:TFont);
begin
   FTextFont.Assign(Value);
   Invalidate;
end;
procedure TOverComp.FontChanged(Sender: TObject);
begin
   Perform(CM_FONTCHANGED, 0, 0);
end;
procedure Register;
begin
  RegisterComponents('i-griegavcl', [TOverComp]);
end;

end.

