unit PictureList;

interface

uses
  Windows, Messages, SysUtils, Classes,graphics,funciones,igriegaclasses;

type
  TPictureItem=class(TCollectionItem)
     private
        FPicture:TPicture;
        FName:String;
        FLink:TLinkClass;
        procedure SetPicture(value:TPicture);
     protected
        function GetDisplayName: string; override;
     public
        constructor Create(Collection:TCollection);override;
        Destructor Destroy;override;
     published
        property Name:String read FName write FName;
        property Picture:TPicture read FPicture Write SetPicture;
        property Link : TLinkClass read fLink write FLink;
  end;
  TPictures=Class(TCollection)
     private
        FOwner:TPersistent;
        function GetItem(Index:Integer):TPictureItem;
        Procedure SetItem(Index:Integer;Value:TPictureItem);
     protected
        function GetOwner:TPersistent;override;
        procedure Notify(Item: TCollectionItem; Action: TCollectionNotification);override;
     public
         Constructor Create(Owner:TPersistent);
         Function Add:TPictureItem;
         property Items[Index:Integer]:TPictureItem read Getitem write SetItem; default;

     published
  end;

  TPictureList = class(TComponent)
  private
     FPictures:TPictures;
     FActive:Boolean;
    { Private declarations }
  protected
    { Protected declarations }
  public
     constructor Create(Aowner:TComponent);override;
     Destructor Destroy;override;
     Function FindIndex(AName:String):Integer;
     Function Picture(AName:String):TPicture;overload;
     Function Picture(AIndex:Integer):TPicture;overload;
    { Public declarations }
  published
     Property Pictures:TPictures read FPictures Write FPictures;
     Property Active: Boolean read FActive write FActive default true;
    { Published declarations }
  end;

procedure Register;

implementation
constructor TPictureList.Create(AOwner:TComponent);
begin
   inherited;
   FPictures:=TPictures.Create(self);
   FActive:=true;
end;
Destructor TPictureList.destroy;
begin
   FPictures.Free;
   inherited;
end;
Function TPictureList.FindIndex(AName:String):Integer;
var
   i:integer;
   found:Boolean;
begin
   found := false;
   for i:=0 to Pictures.Count-1 do
   begin
      if uppercase(Pictures[i].Name)=uppercase(AName) then
      begin
         found:=true;
         break;
      end;
   end;
   if found then result:=i else result:=-1;
end;
Function TPictureList.Picture(AName:String):TPicture;
var
   i:integer;
   found:Boolean;
begin
   found := false;
   for i:=0 to Pictures.Count-1 do
   begin
      if uppercase(Pictures[i].Name)=uppercase(AName) then
      begin
         found:=true;
         break;
      end;
   end;
   if found then result:=Pictures[i].picture else result:=nil;
end;
Function TPictureList.Picture(AIndex:Integer):TPicture;
begin
   if between(AIndex,0,Pictures.Count-1) then
     result:=Pictures[AIndex].picture
   else result:=nil;
end;

constructor TPictureItem.Create(Collection:TCollection);
begin
   inherited;
   FPicture:=Tpicture.create;
   FName:='Picture'+inttostr(index);
   FLink:=TLinkClass.Create;
end;
Destructor TPictureItem.destroy;
begin
   fpicture.Free;
   Flink.Free;
   inherited;
end;
procedure TPictureItem.SetPicture(Value:TPicture);
begin
   fpicture.Assign(Value);
end;
function TPictureItem.GetDisplayName: string;
begin
   result:=Name;
end;
constructor TPictures.Create(Owner:TPersistent);
begin
   inherited Create(TPictureItem);
   Fowner:=owner;
end;
procedure TPictures.Notify(Item: TCollectionItem; Action: TCollectionNotification);
begin
//   if action=cnDeleting then showmessage('Borrar');
   inherited Notify(Item, Action);
end;
function TPictures.GetItem(Index:Integer):TPictureItem;
begin
   result:=TPictureItem(inherited GetItem(Index));
end;
Procedure TPictures.SetItem(Index:Integer;Value:TPictureItem);
begin
   inherited SetItem(Index,Value);
end;
Function TPictures.GetOwner:Tpersistent;
begin
   result:=Fowner;
end;
Function TPictures.Add:TPictureItem;
begin
   result:=TPictureItem(Inherited Add);
end;
procedure Register;
begin
  RegisterComponents('i-vcl Non-Visual', [TPictureList]);
end;

end.
