unit ShapeForm;

interface

uses
  Windows, Messages, SysUtils, Classes,forms,
  GRAPHICS,igriegastrs,igriegaclasses,funciones;
const
  MaxNumberPoints=1000;
type
  Tpar = record
    x,y : integer;
  end;
  TCombineMode=(cmAND,cmDIFF,cmOR,cmXOR);
const
  ACombineMode:Array[cmAND..cmXOR] of integer=(RGN_AND,RGN_DIFF,RGN_OR,RGN_XOR);
type
  TShapePolygon=Class(TCustomPolygons)
     private
        FNCPolygon:Integer; //Non-Client Polygon
        FCombineMode:TCombineMode;
        function GetNCPolygon:Integer;
        procedure SetNCPolygon(Value:Integer);
        procedure SetCombineMode(Value:TCombineMode);
     protected
        procedure Update(Item: TCollectionItem); override;
     public
        Constructor Create(Owner:TPersistent);
     published
        property NCPolygon:Integer read GetNCPolygon Write SetNCPolygon;
        property CombineMode:TCombineMode read FCombineMode write SetCombineMode default cmOR;
        property LoadType;
  end;
  TPerColors=Class(TPersistent)
     private
        FTransparent,
        FTolerance:TColor;
        FOnChange:TNotifyEvent;
        procedure SetTransparent(Value:TColor);
        procedure SetTolerance(Value:TColor);
     protected
     public
        constructor Create;
        property OnChange:TNotifyEvent read FOnChange Write FOnChange;
     published
        property Transparent:TColor read FTransparent Write SetTransparent default clWhite;
        property Tolerance:TColor read FTolerance Write SetTolerance default clWhite;
  end;
  TFramed=Class(TCustomFramed)
     public
        property OnChange;
     published
        property Brush;
        property Size;
  end;
  TShapes=(sfRect,sfElliptical,sfImage,sfRounredRect,sfPolygon);
  TOnShapeChangeEvent=procedure(Sender:TObject;Var NewShape:TShapes) of object;
  TOnStatusChangeEvent=procedure(Sender:TObject;Var Status:Boolean) of object;
  TShapeForm = class(TComponent)
  private
     FUpdatePol:Boolean;
     FActive:Boolean;
     FShape:TShapes;
     FPicture:TBitmap;
     FOnPaint:TNotifyEvent;
     FCorner:TPointType;
     FPoints:TShapePolygon;
     FOnShapeChange:TOnShapeChangeEvent;
     FOnStatusChange:TOnStatusChangeEvent;
     FDraw:Boolean;
     FColors:TPerColors;
     FWindowProc:TWndMethod;
     FNewShape:HRGN;
     FFrame:TFramed;
     FFramed:Boolean;
     FBorderStyle:TFormBorderStyle;
     FBorderWidth:Integer;
     procedure SetCorner(Value:TPointType);
     procedure SetActive(Value:Boolean);
     procedure SetShape(Value:TShapes);
     procedure SetPicture(Value:TBitmap);
     procedure paint(Sender:TObject);
     procedure NewWindowProc(var Msg:TMessage);
     procedure SetFramed(Value:Boolean);
    { Private declarations }
  protected
      procedure PictureChange(Sender:TObject);
      procedure Change(Sender:TObject);
      procedure Invalidate;
      procedure DrawWindow;
    { Protected declarations }
  public
     Constructor Create(AOwner:TComponent);override;
     destructor destroy;override;
     procedure DrawFrame;
    { Public declarations }
  published
     property Corner:TPointType read FCorner Write SetCorner;
     property Active:Boolean read FActive Write SetActive default False;
     property Shape:TShapes read FShape Write SetShape default sfRect;
     property Picture:TBitmap read FPicture Write SetPicture;
     property Polygons:TShapePolygon read FPoints Write FPoints;
     property DrawPicture:Boolean read FDraw Write FDraw default true;
     property Color:TPerColors read FColors Write FColors;
     property Frame:TFramed read FFrame Write FFrame;
     property Framed:Boolean read FFramed Write SetFramed default False;
     property FormBorderStyle:TFormBorderStyle read FBorderStyle write FBorderStyle default bsSizeable;
     property FormBorderWidth:Integer read FBorderWidth write FBorderWidth default 0;
     property OnShapeChange:TOnShapeChangeEvent read FOnShapeChange write FOnShapeChange;
     property OnStatusChange:TOnStatusChangeEvent read FOnStatusChange write FOnStatusChange;
    { Published declarations }
  end;

procedure Register;

implementation
{$R-}
Constructor TShapePolygon.Create(Owner:TPersistent);
begin
   inherited;
   FNCPolygon:=-1;
   FCombineMode:=cmOR;
end;
Procedure TShapePolygon.Update(Item:TCollectionItem);
begin
   TShapeForm(Owner).Invalidate;
end;
function TShapePolygon.GetNCPolygon:Integer;
begin
   Result:=FNCPolygon;
end;
procedure TShapePolygon.SetNCPolygon(Value:Integer);
begin
   if Value<>FNCPolygon then
      FNCPolygon:=Value;
end;
procedure TShapePolygon.SetCombineMode(Value:TCombineMode);
begin
   if FCombineMode<>Value then
   begin
      FCombineMode:=Value;
      TShapeForm(Owner).Invalidate;
   end;
end;

constructor TPerColors.Create;
begin
   FTransparent:=clWhite;
   FTolerance:=clWhite;
end;
//-----------------------------------------------
// ShapeForm ------------------------------------
//-----------------------------------------------
Constructor TShapeForm.Create(AOwner:TComponent);
var
   i:Integer;
begin
   FUpdatePol:=true;
   for i:=0 to TForm(AOwner).ComponentCount-1 do
      if TForm(AOwner).Components[i].ClassType=TShapeForm then
      begin
         raise Exception.Create(sMsgShapeFormExist);
      end;
   inherited;
   FActive:=False;
   FShape:=sfRect;
   FPicture:=TBitmap.Create;
   FPicture.OnChange:=PictureChange;
   FCorner:=TPointType.Create;
   FCorner.OnChange:=Change;
   FPoints:=TShapePolygon.Create(self);
   FPoints.NCPolygon:=-1;
   FDraw:=True;
   FCorner.SetPoint(point(100,100));
   FColors:=TPercolors.Create;
   FColors.OnChange:=Change;
   FFrame:=TFramed.Create;
   FFramed:=False;
   FBorderStyle:=bsSizeable;;
   FBorderWidth:=0;
   if not (csDesigning in ComponentState) then
   begin
      FOnPaint:=TForm(AOwner).OnPaint;
      TForm(AOwner).OnPaint:=paint;
      FWindowProc:=TForm(AOwner).WindowProc;
      TForm(AOwner).WindowProc:=NewWindowProc;
      FFrame.OnChange:=Change;
   end;
end;
destructor TShapeForm.destroy;
begin
   FCorner.Free;
   FPoints.Free;
   FFrame.Free;
   if not (csDesigning in ComponentState) then
   begin
      if Assigned(FOnPaint) then
         TForm(Owner).OnPaint:=FOnPaint;
      FOnPaint:=nil;
   end;
   FPicture.Free;
   inherited;
end;
procedure TShapeForm.SetFramed(Value:Boolean);
begin
   if FFramed<>Value then
   begin
      FFramed:=Value;
      Invalidate;
   end;
end;
procedure TShapeForm.NewWindowProc(var Msg:TMessage);
var
  Arr: Array[0..MaxNumberPoints] of tpoint;
  n:Integer;
  rgn:HRGN;
  r:TRect;
  t:TPoint;
begin
   if (Msg.Msg=WM_NCHitTest) and (Self.Polygons.NCPolygon>-1) and(Self.Polygons.NCPolygon<=Self.Polygons.Count-1) then
   begin
      case Polygons[Polygons.NCPolygon].PolType of
         ptOther:
            begin
               Self.Polygons[Polygons.NCPolygon].Points.ArrPolygon(Arr);
               n:=Self.Polygons[Polygons.NCPolygon].Points.Count;
               rgn:=CreatePolygonRgn(arr,n,APolyFillMode[Polygons[Polygons.NCPolygon].PolyFillMode]);
            end;
         ptRect:
            begin
               r:=Polygons[Polygons.NCPolygon].Points.rectangle;
               rgn:=createrectrgnindirect(R);
            end;
         ptRoundRect:
            begin
               r:=Polygons[Polygons.NCPolygon].Points.rectangle;
               rgn:=CreateRoundRectRgn(r.Left,r.Top,r.Right,r.Bottom,
                       Polygons[Polygons.NCPolygon].Corners.X,
                       Polygons[Polygons.NCPolygon].Corners.y);
            end;
         ptOval:
            begin
               r:=Polygons[Polygons.NCPolygon].Points.rectangle;
               rgn:=createellipticrgnindirect(R);
            end;
      end;
      t:=tform(owner).screentoclient(point(TWMNCHitTest(Msg).XPos,TWMNCHitTest(Msg).YPos));
      if PtInRegion(rgn,t.X,t.Y) then
         msg.result := HTCAPTION
      else
         FWindowProc(Msg);
      deleteobject(rgn);
   end
   else
      if Msg.Msg=WM_SIZE then
      begin
         FWindowProc(Msg);
         Invalidate;
      end
      else
      FWindowProc(Msg);
end;

procedure TShapeForm.Invalidate;
begin
   if FUpdatePol then
      DrawWindow;
end;
procedure TShapeForm.SetPicture(Value:TBitmap);
begin
   FPicture.Assign(Value);
   if Shape=sfImage then
      Invalidate;
end;
procedure TShapeForm.Change(Sender:TObject);
begin
   Invalidate;
end;
procedure TShapeForm.SetCorner(Value:TPointType);
begin
   if (FCorner.x<>Value.X) or (FCorner.Y<>Value.Y) then
   begin
      FCorner:=Value;
      Invalidate;
   end;
end;
procedure TShapeForm.Paint(Sender:TObject);
var
   r:HRGN;
   B:TBrush;
begin
   if Assigned(fonpaint) then FOnPaint(Sender);
   if (Shape=sfImage) and DrawPicture then
      TForm(Sender).Canvas.Draw(0,0,Picture);
   if Framed then
   begin
      r:=CreateRectRgn(0,0,0,0);
      GetWindowRgn(TForm(Owner).Handle,r);
      B:=TBrush.Create;
      TForm(Owner).Canvas.Brush:=Frame.Brush;
      FrameRgn(TForm(Owner).Canvas.Handle,r,TForm(Owner).Canvas.Brush.Handle,Frame.Size,Frame.Size);
      B.Free;
      DeleteObject(r);
   end;
end;
procedure TPerColors.SetTransparent(Value:TColor);
begin
  if FTransparent<>Value then
  begin
     FTransparent:=Value;
     if Assigned(FOnchange) then FOnChange(Self);
  end;
end;
procedure TPerColors.SetTolerance(Value:TColor);
begin
  if FTolerance<>Value then
  begin
     FTolerance:=Value;
     if Assigned(FOnchange) then FOnChange(Self);
  end;
end;
procedure TShapeForm.PictureChange(Sender:TObject);
begin
   if Shape=sfImage then
      Invalidate;
end;
procedure TShapeForm.SetShape(Value:TShapes);
var
   V:TShapes;
begin
  if FShape<>Value  then
  begin
     v:=Value;
     if assigned(FOnShapeChange) then
        FOnShapeChange(self,V);
     FShape:=V;
     Invalidate;
  end;
end;
procedure TShapeForm.SetActive(Value:Boolean);
var
   v:Boolean;
begin
   if FActive<>Value then
   begin
      v:=value;
      if assigned(FOnStatusChange) then
         FOnStatusChange(self,v);
      FActive:=V;
      if FActive then
      begin
         TForm(Owner).BorderStyle:=bsNone;
         TForm(Owner).BorderWidth:=0;
      end
      else
      begin
         TForm(Owner).BorderStyle:=FormBorderStyle;
         TForm(Owner).BorderWidth:=FormBorderWidth;
      end;
      Invalidate;
   end;
end;
procedure TShapeForm.DrawWindow;
var
  R:TRect;
  Arr:array[0..MaxNumberPoints] of tpoint;
  N,npoints:Integer;
  i,j:Integer;
  SRgn:HRGN;
  function CombMode(A:Integer):Integer;
  begin
     if A=0 then
        result:=RGN_OR
     else
        result:=aCombinemode[polygons.combinemode];
  end;
begin
   if Active then
   begin
      R:=TForm(Owner).Clientrect;
      DeleteObject(FNewShape);
      case Shape of
         sfElliptical:Fnewshape:= CreateEllipticRgnIndirect(R);
         sfImage:
            begin
                Fnewshape:=BitmapToRegion(Picture.Handle,Color.Transparent,color.Tolerance);
            end;
         sfRounredRect: FNewShape:=CreateRoundRectRgn(R.Left,R.Top,R.Right,R.Bottom,Corner.X,Corner.Y);
         sfPolygon:
            begin
               FNewshape:=createrectrgn(0,0,0,0);
               FUpdatePol:=False;
               if not polygons.Loading then
               begin
               if polygons.Count=0 then
                  polygons.Add;
               if ((Polygons[0].Points.Count<3) and (Polygons[0].PolType=ptOther)) or (Polygons[0].Points.Count<2) then
               begin
                  Polygons[0].Points.Clear;
                  Polygons[0].Points.Add(Point(R.left,R.Top));
                  Polygons[0].Points.Add(Point(R.Right,R.Top));
                  Polygons[0].Points.Add(Point(R.Right,R.Bottom));
                  Polygons[0].Points.Add(point(R.Left,r.Bottom));
               end;
               end;
               FUpdatePol:=True;
               N:= Polygons.Count;
               for i:=0 to N-1 do
               begin
                  case Polygons[i].PolType of
                     ptOther:
                        begin
                           for j:=0 to Polygons[i].Points.count-1 do
                              Arr[j]:=polygons[i].Points[j].GetPoint;
                           npoints:=Polygons[i].Points.count;
                           SRgn:=createpolygonrgn(Arr,npoints,APolyFillMode[Polygons[i].PolyFillMode]);
                           combinergn(FNewShape,FNewshape,sRgn,combmode(i));
                           deleteobject(sRGN);
                        end;
                     ptRect:
                        begin
                           r:=Polygons[i].Points.rectangle;
                           SRgn:=createrectrgnindirect(R);
                           combinergn(FNewShape,FNewshape,sRgn,combmode(i));
                           deleteobject(sRGN);
                        end;
                     ptRoundRect:
                        begin
                           r:=Polygons[i].Points.rectangle;
                           SRgn:=CreateRoundRectRgn(r.Left,r.Top,r.Right,r.Bottom,
                                 Polygons[i].Corners.X,
                                 Polygons[i].Corners.Y);;
                           combinergn(FNewShape,FNewshape,sRgn,combmode(i));
                           deleteobject(sRGN);
                        end;
                     ptOval:
                        begin
                           r:=Polygons[i].Points.rectangle;
                           SRgn:=CreateEllipticRgnIndirect(R);
                           combinergn(FNewShape,FNewshape,sRgn,combmode(i));
                           deleteobject(sRGN);
                        end;
                  end;
               end;
            end;
      else
         Fnewshape:= CreateRectRgnIndirect(R);
      end;
      if csDesigning in ComponentState then
         OffsetRgn(FNewShape,getsystemmetrics(SM_CXBORDER)+GetSystemMetrics(SM_CXEDGE),+GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYEDGE)+GetSystemMetrics(SM_CYBORDER));
      setwindowrgn(Tform(owner).handle,Fnewshape,TRUE);
      if (Shape=sfImage) and DrawPicture then
         TForm(Owner).Repaint;
   end
   else
   begin
      R:=TForm(Owner).Clientrect;
      setwindowrgn(Tform(owner).handle,0,TRUE);
   end;
end;
procedure TShapeForm.DrawFrame;
var
   r:HRGN;
   B,BTMP:TBrush;
begin
   r:=CreateRectRgn(0,0,0,0);
   GetWindowRgn(TForm(Owner).Handle,r);
   Btmp:=TBrush.Create;
   btmp.Assign(TForm(Owner).Canvas.Brush);
   B:=TBrush.Create;
   TForm(Owner).Canvas.Brush:=Frame.Brush;
   FrameRgn(TForm(Owner).Canvas.Handle,r,TForm(Owner).Canvas.Brush.Handle,Frame.Size,Frame.Size);
   TForm(Owner).Canvas.Brush:=BTMP;
   B.Free;
   BTMP.Free;
   DeleteObject(r);
end;
procedure Register;
begin
  RegisterComponents('i-vcl Non-Visual', [TShapeForm]);
end;

end.
