unit Tick;

interface

uses
  Windows, Messages, SysUtils, Classes,dateutils;

type
  TTickNote=record
     Tick:Int64;
     Value:String;
  end;
  TTickSourceType=(tsInterval,tsSystem);
  TTypeAllNotes=(anMilliseconds,anSeconds,anMinutes,anHours);
  TNoteType=Class(TPersistent)
     private
        FTickSeparator,
        FNoteSeparator:String;
        FTickSource:TTickSourceType;
        FTickFormat:TTypeAllNotes;
        FClear:Boolean;
     protected
     public
        constructor Create;
     published
        property TickSeparator:String read FTickSeparator Write FTickSeparator;
        property NoteSeparator:String read FNoteSeparator Write FNoteSeparator;
        property TickSource:TTickSourceType read FTickSource write FTickSource default tsInterval;
        property TickFormat:TTypeAllNotes read FTickFormat write FTickFormat default anMilliseconds;
        property Clear:Boolean read FClear Write FClear default true;
  end;
  TTick = class(TComponent)
  private
     FInitial,
     FFinal:Int64;
     FActive:Boolean;
     FList:TStringList;
     FOnRun,
     FOnStop,
     FOnClick:TNotifyEvent;
     FNotes:TNoteType;
     Function GetInitial:Int64;
     Function GetFinal:Int64;
    { Private declarations }
  protected
    { Protected declarations }
  public
     constructor Create(Aowner:TComponent); override;
     destructor Destroy;override;
     Procedure Run;                             // Mark Tick-In
     Procedure Stop;                            // Mark Tick-Out
     Procedure Click;                           // New Mark Tick-In
     procedure Note(Value:String='');           // Add Note of Current Tick
     Function  GetNote(Value:Integer):TTickNote; // return Note[Value]
     Function GetAllNotes:String;
     Procedure DeleteNote(Value:Integer);       // Detete Note[Value]
     procedure ClearNotes;
     Function  NotesCount:Integer;               // Number of Notes
     function  Current:Int64;                    // Current Tick
     Function  Interval:Int64;                   // Time between FInitial and Current Time or FFinal
     Function  AsSeconds(Value:Int64=-1):double;  // Interval/Value as Seconds
     Function  AsIntSeconds(Value:Int64=-1):Int64;
     Function  AsMinutes(Value:Int64=-1):double; // Interval/Value as Minutes
     Function  AsIntMinutes(Value:Int64=-1):Int64;
     Function  AsHours(Value:Int64=-1):double;   // Inteval/Value as Hours
     Function  AsIntHours(Value:Int64=-1):Int64;
     function  AsDateTime(Value:Int64=-1):TDateTime; // Interval/Value in DateTime format
     function  AsTimeStamp(Value:Int64=-1):TTimeStamp; // Interval/Value in DateTime format
     property  Active:Boolean read FActive default false;
     Property  Initial:Int64 read GetInitial;  // Initial Tick
     Property  Final:Int64 read GetFinal;      // Final Tick
  published
     property Notes:TNoteType read FNotes write FNotes;
     Property OnRun:TNotifyEvent read FOnRun write FOnRun;
     Property OnStop:TNotifyEvent read FOnStop write FOnStop;
     Property OnClick:TNotifyEvent read FOnClick write FOnClick;
    { Public declarations }
  end;
procedure Register;
implementation
constructor TNoteType.Create;
begin
   inherited;
   FTickSeparator:='#';
   FNoteSeparator:='@';
   FTickSource:=tsInterval;
   FTickFormat:=anMilliseconds;
   FClear:=true;
end;
constructor TTick.Create;
begin
   inherited;
   FNotes:=TNoteType.Create;
   FList:=TStringList.Create;
   FList.Clear;
   FInitial:=0;
   FFinal:=0;
   FActive:=False;
end;
destructor TTick.Destroy;
begin
   FList.Free;
   FNotes.Free;
   inherited;
end;
Procedure TTick.Run;
begin
   if not Active then
   begin
      FInitial:=gettickcount;
      FFinal:=0;
      FActive:=true;
      if Notes.Clear then
         Flist.Clear;
      if notes.TickSource=tsSystem then
         FList.Add(inttostr(FInitial)+';RUN')
      else
         FList.Add('0;RUN');
      if assigned(FOnRun) then FOnRun(Self);
   end;
end;
Procedure TTick.Click;
begin
   if Active then
   begin
      FInitial:=gettickcount;
      if notes.TickSource=tsSystem then
         FList.Add(inttostr(FInitial)+';CLICK')
      else
         FList.Add('0;CLICK');
      if assigned(FOnClick) then FOnClick(Self);
   end;
end;
Procedure TTick.Stop;
begin
   if Active then
   begin
      FFinal:=gettickcount;
      FActive:=false;
      if notes.TickSource=tsSystem then
         FList.Add(inttostr(FFinal)+';STOP')
      else
         FList.Add(inttostr(Interval)+';STOP');
      if assigned(FOnStop) then FOnStop(Self);
   end;
end;
Function TTick.Interval:Int64;
var
   v:Int64;
begin
   if Active then
      V:=gettickcount-fInitial
   else
      v:=FFinal-FInitial;
   result:=v;
end;
Function TTick.AsSeconds(Value:Int64=-1):Double;
begin
   if value<0 then
      Result:=Interval/1000
   else
      result:=Value/1000;
end;
Function TTick.AsIntSeconds(Value:Int64=-1):Int64;
begin
   if value<0 then
      Result:=Interval div 1000
   else
      result:=Value div 1000;
end;
Function TTick.AsMinutes(Value:Int64=-1):Double;
begin
   if value<0 then
      Result:=Interval/60000
   else
      result:=Value/60000;
end;
Function TTick.AsHours(Value:Int64=-1):Double;
begin
   if value<0 then
      Result:=Interval/3600000
   else
      result:=Value/3600000;
end;
Function TTick.AsIntMinutes(Value:Int64=-1):Int64;
begin
   if value<0 then
      Result:=Interval div 60000
   else
      result:=Value div 60000;
end;
Function TTick.AsIntHours(Value:Int64=-1):Int64;
begin
   if value<0 then
      Result:=Interval div 3600000
   else
      result:=Value div 3600000;
end;
function TTick.AsDateTime(Value:Int64=-1):TDateTime;
var
   a:TTimeStamp;
begin
   if value<0 then
   begin
      a:=MSecsToTimeStamp(Interval);
      a.Date:=1;
      result:=TimeStampToDateTime(a)-1
   end
   else
   begin
      a:=MSecsToTimeStamp(Value);
      a.Date:=1;
      result:=TimeStampToDateTime(a)-1;
   end;
end;
function  TTick.AsTimeStamp(Value:Int64=-1):TTimeStamp;
begin
   if value<0 then
      result:=MSecsToTimeStamp(Interval)
   else
      result:=MSecsToTimeStamp(Value);
end;
Function TTick.GetInitial:Int64;
begin
   if FInitial=0 then
      result:=-1
   else
      result:=FInitial;
end;
Function TTick.GetFinal:Int64;
begin
   if FFinal=0 then
      result:=-1
   else
      result:=FFinal;
end;
procedure TTick.Note(Value:String='');
var
   VTick:String;
   ITick:Int64;
begin
   if active then
   begin
      if notes.TickSource=tsSystem then
         ITick:=gettickcount
      else
         ITick:=Interval;
      if Value='' then
         FList.Add(Inttostr(ITick))
      else
      begin
         VTick:=Inttostr(ITick);
         FList.Add(VTick+';'+Value);
      end;
   end;
end;
Function TTick.NotesCount:Integer;
begin
   result:=FList.Count;
end;
Function TTick.GetNote(Value:Integer):TTickNote;
var
   n:integer;
begin
   if (value>=0) and (value<=FList.Count-1) then
   begin
      n:=pos(';',FList[Value]);
      if n>0 then
      begin
         result.Tick:=strtoint64(copy(FList[value],1,n-1));
         result.Value:=copy(FList[value],n+1,length(FList[value]));
      end
      else
      begin
         result.Tick:=strtoint64(FList[value]);
         result.Value:='';
      end;
   end
   else
      begin
         result.Tick:=-1;
         result.Value:='';
      end;
end;
procedure TTick.ClearNotes;
begin
   FList.Clear;
end;
Function TTick.GetAllNotes:String;
var
   i:Integer;
   n:integer;
   function Translate:String;
   var
      temp:Int64;
   begin
      temp:=strtoint64(copy(FList[i],1,n-1));
      case Notes.TickFormat of
         anMilliSeconds:result:=inttostr(temp);
         anSeconds:result:=inttostr(AsIntSeconds(temp));
         anMinutes:result:=inttostr(AsIntMinutes(temp));
         anHours:result:=inttostr(AsIntHours(temp));
      else
         result:=Inttostr(temp);
      end;
   end;
begin
   if FList.Count>0 then
   begin
      for i:=0 to FList.Count-1 do
      begin
         n:=pos(';',FList[i]);
         if n>0 then
            result:=result+translate+notes.TickSeparator+copy(FList[i],n+1,length(FList[i]))+notes.NoteSeparator
         else
            result:=result+translate+notes.TickSeparator+notes.NoteSeparator;
      end;
   end
   else
      result:='';
end;
Procedure TTick.DeleteNote(Value:Integer);
begin
   FList.Delete(Value);
end;
function TTick.Current:Int64;
begin
   result:=gettickcount;
end;
procedure Register;
begin
   RegisterComponents('i-vcl Non-Visual', [TTick]);
end;
end.
