unit Visualprop;

interface

uses
  Windows, Messages, SysUtils, Classes,dialogs,forms,typinfo,Graphics,controls;
const
  AcceptAssign:set of TTypeKind=[tkInteger, tkChar, tkEnumeration, tkFloat,
  tkString, tkSet, tkWChar, tkLString, tkWString, tkVariant, tkArray, tkRecord, tkInt64, tkDynArray];

type
  TVisupropItem=Class(TCollectionItem)
     private
        FProp:ShortString;
        FValue:variant;
     protected
        function GetDisplayName:string;override;
     public
        Constructor Create(Collection:TCollection);override;
     published
        Property Prop:ShortString read FProp write FProp;
        Property Value:variant read FValue write FValue;
  end;
  TVisuPropCol=Class(TCollection)
     private
        FOwner:TPersistent;
        Function GetItem(Index:Integer):TVisupropItem;
        Procedure SetItem(Index:Integer;Value:TVisupropItem);
     protected
        Function GetOwner:TPersistent;override;
     public
        Constructor Create(Aowner:TPersistent);
        Function Add:TVisupropItem;
        Function FindIndex(Text:String):Integer;
        Procedure SetPropValue(Propertie:string;Value:Variant);
        Property Items[Index:Integer]:TVisupropItem read GetItem Write SetItem;default;
     published
  end;

  TVisualprop = class(TComponent)
  private
    FProperties:TVisuPropCol;
    FTagValue:Integer;
    FFont:TFont;
    FUseFont:Boolean;
    FComp:TComponent;
    FFileName:TFilename;
    FListElements:TStringList;
    procedure SetGetFrom(Value:TComponent);
    procedure SetFilename(Value:TFilename);
    function GetElements:TStringList;
    { Private declarations }
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    Procedure Perform;
    Procedure SaveToFile(F:Tfilename);
    Procedure LoadFromFile(F:TFilename);
    Procedure PropFrom(Component:TComponent);overload;
    Procedure PropFrom(Component:TComponent;properties:array of string);overload;
    Procedure PropFromArray(properties:array of string;values:array of variant);
    { Public declarations }
  published
    Property Properties:TVisuPropCol read FProperties Write FProperties;
    Property TagValue:Integer read FTagValue write FTagValue default -1;
    Property Font:TFont Read FFont write FFont;
    Property UseFont:Boolean read FUseFont Write FUseFont default false;
    Property GetFrom:TComponent read FComp Write SetGetFrom;
    property FileName:TFilename read FFilename write SetFileName;
    property Elements:TStringList read GetElements;
    { Published declarations }
  end;

implementation
constructor TVisualprop.Create(AOwner:TComponent);
begin
   inherited;
   FProperties:=TVisuPropCol.Create(Self);
   FFont:=TFont.create;
   FUseFont:=false;
   FFilename:='';
   FListElements:=tstringlist.Create;
   FTagValue:=-1;
end;
Destructor TVisualprop.Destroy;
begin
   FProperties.Free;
   FFont.Free;
   FLIstElements.Free;
   inherited;
end;
function TVisualprop.GetElements:TStringList;
var
   i:integer;
begin
   flistelements.Clear;
   for i:=0 to Owner.ComponentCount-1 do
      if Owner.Components[i].Tag=FTagValue then
         flistelements.Add(Owner.Components[i].Name+' ['+Owner.Components[i].ClassName+']');
   result:=flistelements;
end;
procedure TVisualprop.Perform;
var
   i,j:integer;
begin
   if self.Owner is TForm then
   begin
      for i:=0 to tform(Owner).ComponentCount-1 do
      begin
         if tform(owner).Components[i].Tag=self.TagValue then
            for j:=0 to self.Properties.Count-1 do
               if GetPropinfo(tform(owner).Components[i],Properties.Items[j].prop)<>nil then
                  setpropvalue(tform(owner).Components[i],Properties.Items[j].prop,Properties.Items[j].value);
         if FUseFont then
            if tform(owner).Components[i].Tag=self.TagValue then
               if GetPropinfo(tform(owner).Components[i],'Font')<>nil then
                  SetObjectProp(tform(owner).Components[i],'Font',FFont);
      end;
   end;
end;
Procedure TVisualprop.PropFrom(Component:TComponent);
var
   proplist:PProplist;
   c,i:integer;
begin
   filename:='';
   FProperties.Clear;
   c:=getproplist(Component,proplist);
   for i:=0 to c-1 do
      if proplist[i].PropType^.Kind in AcceptAssign then
         if pos(''+uppercase(proplist[i].Name)+'','NAMETAG')=0 then
         begin
            FProperties.Add;
            FProperties.Items[FProperties.Count-1].prop:=proplist[i].Name;
            FProperties.Items[FProperties.Count-1].Value:=getpropvalue(Component,proplist[i].Name);
         end;
end;
Procedure TVisualprop.PropFromArray(properties:array of string;values:array of variant);
var
   i:integer;
begin
   if high(properties)=high(values) then
   begin
      FProperties.Clear;
      for i:=low(properties) to high(properties) do
         if pos(''+uppercase(properties[i])+'','NAMETAG')=0 then
            begin
               FProperties.Add;
               FProperties.Items[FProperties.Count-1].prop:=properties[i];
               FProperties.Items[FProperties.Count-1].Value:=Values[i];
            end;
   end;
end;
Procedure TVisualprop.PropFrom(Component:TComponent;properties:array of string);
var
   propinfo:PPropinfo;
   i:integer;
begin
   filename:='';
   FProperties.Clear;
   for i:=low(properties) to high(properties) do
      if pos(''+uppercase(properties[i])+'','NAMETAG')=0 then
      begin
         propinfo:=getpropinfo(component,properties[i],AcceptAssign);
         if propinfo<>nil then
         begin
            FProperties.Add;
            FProperties.Items[FProperties.Count-1].prop:=properties[i];
            FProperties.Items[FProperties.Count-1].Value:=getpropvalue(Component,properties[i]);
         end;
      end;
end;

procedure TVisualprop.SetGetFrom(Value:TComponent);
begin
   if MessageDlg('Cargar propiedades de '+value.Name+'?',mtConfirmation, [mbYes, mbNo], 0)=mrYes then
   begin
      PropFrom(Value);
   end;

end;
procedure TVisualprop.SaveToFile(F: Tfilename);
var
   T:TStringlist;
   i:integer;
   s:string;
begin
  t:=tstringlist.Create;
  if UseFont then
     t.Add('UseFont=True')
  else
     t.Add('UseFont=False');
  for i:=0 to FProperties.Count-1 do
  begin
     s:=FProperties[i].FValue;
     t.Add(FProperties[i].FProp+'='+s);
  end;
  t.SaveToFile(F);
  t.Free;
end;
procedure TVisualprop.SetFilename(value:TFilename);
begin
   if value<>'' then
   begin
      if value<>FFilename then
         LoadFromFile(value);
   end;
   FFilename:=value
end;
Procedure TVisualprop.LoadFromFile(F:TFilename);
var
   T:TStringlist;
   i:integer;
begin
   T:=TStringlist.Create;
   t.LoadFromFile(F);
   Fproperties.Clear;
   for i:=0 to t.Count-1 do
      if pos('=',t[i])>0 then
      begin
         if trim(Uppercase(copy(t[i],1,pos('=',t[i])-1)))='USEFONT' then
         begin
            if UpperCase(trim(copy(t[i],pos('=',t[i])+1,length(t[i]))))='TRUE' then
               UseFont:=true
            else
               UseFont:=False;
         end
         else
         begin
            FProperties.Add;
            FProperties[FProperties.Count-1].FProp:=trim(copy(t[i],1,pos('=',t[i])-1));
            FProperties[FProperties.Count-1].FValue:=trim(copy(t[i],pos('=',t[i])+1,length(t[i])));
         end;
      end;
end;
constructor TVisupropItem.Create(Collection:TCollection);
begin
   Inherited;
   FProp:='';
   FValue:='';
end;
Function TVisuPropCol.FindIndex(Text:String):Integer;
var i:integer;
begin
   result:=-1;
   for i:=0 to Count-1 do
      if uppercase(items[i].FProp)=uppercase(Text) then
      begin
         result:=i;
         break;
      end;
end;
Procedure TVisuPropCol.SetPropValue(Propertie:string;Value:Variant);
var i:integer;
begin
   i:=findIndex(Propertie);
   if i>=0 then
      items[i].FValue:=value;
end;
function TVisupropItem.GetDisplayName:string;
begin
   result:=FProp;
   if result='' then Result:= inherited GetDisplayName;
end;
constructor TVisuPropCol.Create(AOwner:TPersistent);
begin
   inherited Create(TVisupropItem);
   FOwner:=Owner;
end;
Function TVisuPropCol.GetItem(Index:Integer):TVisupropItem;
begin
   result:=TVisupropItem(inherited GetItem(index));
end;
Procedure TVisuPropCol.SetItem(Index:Integer;Value:TVisupropItem);
begin
   inherited SetItem(Index,Value);
end;
Function TVisuPropCol.GetOwner:TPersistent;
begin
   result:=FOwner;
end;
Function TVisuPropCol.Add:TVisupropItem;
begin
   result:=TVisupropItem(Inherited Add);
end;


end.
