unit FramedPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls,Graphics,math,igriegaClasses;

type
  TFramedPanelType=Class(TPersistent)
  private
     FColor:TColor;
     FSize:integer;
     FAfter:Integer;
     FOnChange:TNotifyEvent;
     procedure SetColor(Value:TColor);
     procedure SetSize(Value:Integer);
     procedure SetAfter(Value:Integer);
  protected
  public
     constructor Create;
  published
     property Color:TColor read FColor write SetColor default clBlack;
     Property Size:Integer read FSize write SetSize default 0;
     Property After:Integer read FAfter write SetAfter default 3;
     property OnChange:TNotifyEvent read FOnChange write FOnChange;
  end;
  TFramedPanel = class(TCustomControl)
  private
     FBorder:TBorderLine;
     FBorderOuter,
     FBorderMiddle,
     FBorderInner:TFramedPanelType;
     FBackground:TBitmap;
     procedure Change(sender:TObject);
     procedure SetBackground(Value:TBitmap);
     procedure CMBorderChanged(var Message: TMessage); message CM_BORDERCHANGED;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Paint;override;
    procedure AdjustClientRect(var Rect: TRect);override;
  public
     Constructor Create(AOwner:TComponent);override;
     Destructor Destroy;override;
    { Public declarations }
  published
    property Border:TBorderLine read FBorder Write FBorder;
    property BorderOuter:TFramedPanelType read FBorderOuter write FBorderOuter;
    property BorderMiddle:TFramedPanelType read FBorderMiddle write FBorderMiddle;
    property BorderInner:TFramedPanelType read FBorderInner write FBorderInner;
    property Background:TBitmap read FBackground write SetBackground;
    property Align;
    property Anchors;
    property AutoSize;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;

    { Published declarations }
  end;

procedure Register;

implementation


Constructor TFramedPanel.Create(AOwner:TComponent);
begin
   inherited;
   ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
     csDoubleClicks, csReplicatable];
   FBorderOuter:=TFramedPanelType.Create;
   FBorderMiddle:=TFramedPanelType.Create;
   FBorderinner:=TFramedPanelType.Create;
   FBorderOuter.OnChange:=change;
   FBorderMiddle.OnChange:=change;
   FBorderinner.OnChange:=change;
   FBorderOuter.Size:=1;
   FBorderMiddle.Size:=5;
   FBorderinner.Size:=1;
   FBackground:=Tbitmap.create;
   FBorder:=TBorderLine.Create(Self);
end;
Destructor TFramedPanel.Destroy;
begin
   FBorderOuter.Free;
   FBorderMiddle.Free;
   FBorderinner.Free;
   Fbackground.free;
   inherited;
end;
procedure TFramedPanel.CMBorderChanged(var Message: TMessage);
begin
   invalidate;
end;
procedure TFramedPanel.SetBackground(Value:TBitmap);
begin
   FBackGround.Assign(Value);
   invalidate;
end;
procedure TFramedPanel.Change(Sender:TObject);
begin
   invalidate;
   realign;
end;
procedure TFramedPanel.Paint;
VAR
   r:TRect;
   procedure DrawBorder(Value:TFramedPanelType);
   var   x,y,w,h:integer;
      procedure rectangleR(Ax0,Ay0,Ax1,Ay1:integer);
      begin
         Canvas.MoveTo(Ax0,Ay0);
         if Border.Top then canvas.LineTo(Ax1,Ay0)
         else Canvas.MoveTo(Ax1,Ay0);
         if Border.Right then canvas.LineTo(Ax1,Ay1)
         else Canvas.MoveTo(Ax1,Ay1);
         if Border.Bottom then canvas.LineTo(Ax0,Ay1)
         else Canvas.MoveTo(Ax0,Ay1);
         if Border.Left then canvas.LineTo(Ax0,Ay0)
         else Canvas.MoveTo(Ax0,Ay0);
      end;
   begin
      if Value.Size>0 then
      with canvas do
      begin
         Pen.Width:=Value.Size;
         Pen.Color:=Value.Color;
         X := r.Left+(Pen.Width div 2);
         Y := X;
         W := (r.Right-r.Left) - Pen.Width + 1;
         H := (r.bottom-r.top) - Pen.Width + 1;
{         if Border.Top and Border.Bottom and Border.Left and border.Right then
            rectangle(x,y,x+w,y+h)
         else}
         rectangleR(x,y,x+w-1,y+h-1);
         r:=rect(x,y,x+w,y+h);
         inflaterect(r,-(value.After+(Pen.Width div 2)),-(value.After+(Pen.Width div 2)));
      end;
   end;
begin
   r:=clientrect;
   with canvas do
   begin
      Brush.Color:=Color;
      Brush.Style:=bsClear;
      DrawBorder(BorderOuter);
      DrawBorder(BorderMiddle);
      DrawBorder(BorderInner);
      Brush.Style:=bsSolid;
      try
         if not Fbackground.Empty then
         begin
            Brush.Bitmap:=FBackground;
            FillRect(R);
         end
         else
            Brush.Bitmap:=nil;
      finally
      end;
   end;
end;
procedure TFramedPanel.AdjustClientRect(var Rect: TRect);
begin
  Rect.Left:=ifthen(BorderOuter.Size>0,BorderOuter.Size+BorderOuter.After-1,0)+
             ifthen(BorderMiddle.Size>0,BorderMiddle.Size+BorderMiddle.After-1,0) +
             ifthen(BorderInner.Size>0,BorderInner.Size+BorderInner.After-1,0);
  Rect.Top:=Rect.Left;
  Rect.Right:=Width-Rect.Left;
  Rect.Bottom:=Height-Rect.Top;
  inherited AdjustClientRect(Rect);
end;
constructor TFramedPanelType.Create;
begin
   FColor:=clBlack;
   FSize:=0;
   FAfter:=3;
end;
procedure TFramedPanelType.SetColor(Value:TColor);
begin
   if value<>FColor then
   begin
      FColor:=Value;
      if assigned(fonchange) then FOnChange(self);
   end;
end;
procedure TFramedPanelType.SetSize(Value:Integer);
begin
   if (value<>FSize) and (value>=0) then
   begin
      FSize:=Value;
      if assigned(fonchange) then FOnChange(self);
   end;
end;
procedure TFramedPanelType.SetAfter(Value:Integer);
begin
   if (value<>FAfter) {and (value>=0)} then
   begin
      FAfter:=Value;
      if assigned(fonchange) then FOnChange(self);
   end;
end;
procedure Register;
begin
  RegisterComponents('i-vcl Panels', [TFramedPanel]);
end;

end.
