{ ****************************************************************** }
{                                                                    }
{   VCL component TLabelButton                                       }
{                                                                    }
{ ****************************************************************** }

unit LabelButton;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics, Stdctrls,ExtCtrls;

type
  TShapeAround=(saNone,saRectangle,saLine,saFrame3DRaised,saFrame3DLowered);
  TLabelButton = class(TCustomLabel)
    private
        FColorOver :TColor;
        FBColorLeave: TColor;
        FOver : Boolean;
        FDown : Boolean;
        FShapeAround:TShapeAround;
        FShapeAroundColor:TColor;
        FFontOver:TFont;
        FFontTmp:TFont;
    protected
        procedure CMMouseLeave(var msg : TMessage); message CM_MOUSELEAVE;
        procedure CMMouseEnter(var msg : TMessage); message CM_MOUSEENTER;
        procedure WMLButtonDown (var Message: TWMLButtonDown);
              message WM_LBUTTONDOWN;
        procedure WMLButtonUp (var Message: TWMLButtonUp);
              message WM_LBUTTONUP;
        procedure paint; override;
    public
        constructor Create(AOwner: TComponent); override;
        Destructor Destroy;override;
    published
        property OnClick;
        property OnDragDrop;
        property OnMouseDown;
        property OnMouseMove;
        property OnMouseUp;
        property Action;
        property FontOver:TFont read FFontOver write FFontOver;
        property ColorOver : TColor Read FColorOver write FColorOver;
        Property ShapeAround:TShapeAround read FShapeAround write FShapeAround default saNone;
        Property ShapeAroundColor:TColor read FShapeAroundColor write FShapeAroundColor;
        property Align;
        property Alignment;
        property Anchors;
        property AutoSize;
        property BiDiMode;
        property Caption;
        property Color;
        property Constraints;
        property DragCursor;
        property DragKind;
        property DragMode;
        property Enabled;
        property Font;
        property ParentBiDiMode;
        property ParentColor;
        property ParentFont;
        property ParentShowHint;
        property PopupMenu;
        property ShowHint;
        property Transparent;
        property Layout;
        property Visible;
        property WordWrap;
  end;

implementation

procedure TLabelButton.CMMouseLeave(var msg : TMessage);
begin
 if fdown then
 begin
   top:=top-1;
   left:=left-1;
   fdown:=false;
 end;
 FOver := False;
 color:=FBcolorLeave;
 Font.Assign(FFonttmp);
 Invalidate;
end;

procedure TLabelButton.CMMouseEnter(var msg : TMessage);
begin
 FOver := True;
 FBColorLeave:=color;
 color:=FColorOver;
 FFonttmp.Assign(font);
 Font.Assign(FFontover);
 Invalidate;
end;


constructor TLabelButton.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FColorOver := Color;
     FShapeAroundColor:=clBlack;
     FShapeAround:=saNone;
     FOver := False;
     cursor:=crHandPoint;
     FfontOver:=tfont.Create;
     FFonttmp:=tfont.Create;
end;
Destructor TLabelButton.Destroy;
begin
   FFontOver.Free;
   Ffonttmp.Free;
   inherited;
end;
procedure TLabelButton.WMLButtondown (var Message: TWMLButtonDown);
begin
   if not fdown then
   begin
      fdown:=true;
      top:=top+1;
      left:=left+1;
   end;
   inherited;
end;
procedure TLabelButton.WMLButtonUp(var Message: TWMLButtonUp);
begin
   if fdown then
   begin
      fdown:=false;
      top:=top-1;
      left:=left-1;
   end;
   inherited;
end;
procedure TLabelButton.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Longint;
  Pencolor:Tcolor;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    if fOver and (FShapeAround<>saNone) then
    begin
       pencolor:=pen.Color;
       Pen.Color := FShapeAroundColor;
       case FShapeAround of
          saRectangle:rectangle(Rect);
          saLine:
             begin
                MoveTo(Rect.Left, rect.Bottom-1);
                lineto(rect.Right,rect.Bottom-1);
             end;
          saFrame3DRaised:frame3d(canvas,rect,clbtnhighlight,clbtnshadow,1);
          saFrame3DLowered:frame3d(canvas,rect,clbtnshadow,clbtnhighlight,1);
       end;
       pen.Color:=pencolor;
    end;
    { DoDrawText takes care of BiDi alignments }
    DrawStyle := DT_EXPANDTABS or WordWraps[WordWrap] or Alignments[Alignment] or DT_NOPREFIX;
    { Calculate vertical layout }
    if Layout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if Layout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    if Fdown and (align<>alNone) then
       OffsetRect(rect,1,1);
    DoDrawText(Rect, DrawStyle);
  end;
end;
end.
