unit StateLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,graphics,funciones;
const
  StateStrings:array[0..5] of string=('BTNEXECUTING','BTNOK','BTNERROR','BTNCANCELED','BTNNONE','BTNWAITING');
type
  TStateValue=(svExecuting,svOk,svError,svCanceled,svNone,svWaiting);
  TStateLabel = class(TCustomLabel)
  private
     FImage:TBitmap;
     FState:TStateValue;
     FOffset:Integer;
     FOnStateChange:TNotifyEvent;
     Procedure SetState(Value:TStateValue);
     Procedure SetOffSet(Value:Integer);
    { Private declarations }
  protected
    procedure Paint;override;
    procedure AdjustBounds;override;
    { Protected declarations }
  public
     Constructor Create(AOwner:TComponent);override;
     Destructor Destroy;override;
    { Public declarations }
  published
    property State:TStateValue read FState write SetState default svNone;
    Property OffSet:Integer read FOffSet write SetOffSet default 1;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
    property OnStateChange:TNotifyEvent read FOnStateChange Write FOnStateChange;
    { Published declarations }
  end;

procedure Register;

implementation
{$R STATEBUTTONS}
constructor TStateLabel.Create(AOwner:TComponent);
var
  tmpimage:TBitmap;
begin
   inherited;
   FImage:=TBitmap.Create;
   FState:=svNone;
   tmpimage:=TBitmap.Create;
   try
     tmpimage.Handle:=LoadBitmap(hinstance,
                              pchar(StateStrings[ord(FState)]));
     FImage.Assign(tmpimage);
     DeleteObject(tmpimage.Handle);
   finally
     tmpimage.Free;
   end;
   iif(height<Fimage.Height,Fimage.Height,height);
   width:=FImage.Width+width;
   FOffSet:=1;
end;
Destructor TStateLabel.Destroy;
begin
   if assigned(FImage) then
      FImage.Free;
   inherited;
end;
procedure TStateLabel.SetOffSet(Value:Integer);
begin
   if (Value>=0) and (FOffSet<>Value) then
   begin
      FOffSet:=Value;
      invalidate;
   end;
end;
procedure TStateLabel.SetState(Value:TStateValue);
var
  tmpimage:TBitmap;
begin
 if FState<>Value then
   begin
      tmpimage:=TBitmap.Create;
      try
        tmpimage.Handle:=LoadBitmap(hinstance,
                                 pchar(StateStrings[ord(value)]));
        FImage.Assign(tmpimage);
        DeleteObject(tmpimage.Handle);
      finally
        tmpimage.Free;
      end;
      FState:=Value;
      invalidate;
      if assigned(FOnstateChange) then
          FOnStateChange(self);
   end;
end;
procedure TStateLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Longint;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    rect.Left:=rect.Left+17;
    { DoDrawText takes care of BiDi alignments }
    DrawStyle := DT_EXPANDTABS or WordWraps[WordWrap] or Alignments[Alignment];
    { Calculate vertical layout }
    if Layout <> tlTop then
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      if Layout = tlBottom then OffsetRect(Rect, 0, Height - CalcRect.Bottom)
      else OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    fimage.Transparent:=true;
    draw(0,(height - Fimage.Height) div 2,Fimage);
    rect.Left:=rect.Left+FOffSet;
    DoDrawText(Rect, DrawStyle);
  end;
end;
procedure TStateLabel.AdjustBounds;
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC: HDC;
  X: Integer;
  Rect: TRect;
  AAlignment: TAlignment;
begin
  if not (csReading in ComponentState) and AutoSize then
  begin
    Rect := ClientRect;
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(Rect, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[WordWrap]);
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    X := Left;
    AAlignment := Alignment;
    if UseRightToLeftAlignment then ChangeBiDiModeAlignment(AAlignment);
    if AAlignment = taRightJustify then Inc(X, Width - Rect.Right);
    if autosize then
    begin
       if fimage.Height>height then
          SetBounds(X, Top, Rect.Right+FImage.Width+FOffset, rect.Bottom+(fimage.Height-height))
       else
          SetBounds(X, Top, Rect.Right+FImage.Width+FOffset, Rect.Bottom);
    end
    else
       SetBounds(X, Top, Rect.Right+FImage.Width+FOffset, Rect.Bottom);
  end;
end;
procedure Register;
begin
  RegisterComponents('i-vcl Labels', [TStateLabel]);
end;

end.
